from importlib import import_module

from django.conf import settings
from django.shortcuts import redirect
from django.utils.deprecation import MiddlewareMixin


class SessionWithThirdPartyMiddleware(MiddlewareMixin):

    def __init__(self, get_response=None):
        self.get_response = get_response
        engine = import_module(settings.SESSION_ENGINE)
        self.SessionStore = engine.SessionStore

    def process_response(self, request, response):
        session_id = request.GET.get('session_id', None)
        if session_id:
            request.session = self.SessionStore(session_id)
            # Should be set before the response is returned, because of the modified session.
            request.session['THIRD_PARTY_LOGIN'] = True
            return redirect(request.path)
        return response



# Generated by Django 2.2 on 2019-06-28 21:32

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Audit',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('action', models.CharField(choices=[('li', 'Action login'), ('lo', 'Action logout'), ('lf', 'Action login failed')], max_length=64, verbose_name='Action')),
                ('ip', models.GenericIPAddressField(null=True, verbose_name='IP')),
                ('extra', models.TextField(blank=True, null=True, verbose_name='Extra data')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='Created at')),
                ('update_at', models.DateTimeField(auto_now=True, verbose_name='Update at')),
                ('user_obj', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL, verbose_name='User')),
            ],
        ),
    ]



"""Default settings for django-notifs project."""

from django.conf import settings


NOTIFICATIONS_PAGINATE_BY = getattr(settings, 'NOTIFICATIONS_PAGINATE_BY', 15)

NOTIFICATIONS_USE_WEBSOCKET = getattr(
    settings, 'NOTIFICATIONS_USE_WEBSOCKET', False
)

NOTIFICATIONS_CHANNELS = getattr(settings, 'NOTIFICATIONS_CHANNELS', {})

if NOTIFICATIONS_USE_WEBSOCKET:
    NOTIFICATIONS_CHANNELS['websocket'] = 'aparnik.contrib.messaging.channels.BasicWebSocketChannel'



# -*- coding: utf-8 -*-

from django.db import models
from django.utils.timezone import now
from django.utils.text import slugify
from django.contrib.auth import get_user_model
from django.utils.translation import ugettext as _

from ckeditor_uploader.fields import RichTextUploadingField
from aparnik.contrib.basemodels.models import BaseModelManager, BaseModel
from aparnik.contrib.filefields.models import FileField
from aparnik.contrib.categories.models import Category

User = get_user_model()


# Create your models here.
class NewsManager(BaseModelManager):
    def get_queryset(self):
        return super(NewsManager, self).get_queryset()

    def this_user(self, user):
        return self.get_queryset().filter(user_obj=user)

    def active(self):
        return self.get_queryset().filter(is_published=True, publish_date__lte=now())


class News(BaseModel):
    user_obj = models.ForeignKey(User, on_delete=models.CASCADE, verbose_name=_('User'))
    title = models.CharField(max_length=200, verbose_name=_('Title'))
    slug = models.SlugField(max_length=300, unique=True, allow_unicode=True, verbose_name=_('Slug'))
    cover_images = models.ManyToManyField(FileField, blank=True, verbose_name=_('Cover Images'))
    content = RichTextUploadingField(config_name='basic', verbose_name=_('Content'))
    is_published = models.BooleanField(default=True, verbose_name=_('Is Published'))
    publish_date = models.DateTimeField(default=now, verbose_name=_('Published Date'))
    categories = models.ManyToManyField(Category, blank=True, verbose_name=_('Categories'))
    # TODO: add tag
    # tag = models.ForeignKey(Category, on_delete=models.CASCADE, verbose_name=_('Tag'))

    objects = NewsManager()

    class Meta:
        verbose_name = _('News')
        verbose_name_plural = _('NEWS')

    def __init__(self, *args, **kwargs):
        super(News, self).__init__(*args, **kwargs)

    def save(self, *args, **kwargs):
        if not self.id:
            self.slug = slugify(self.title, allow_unicode=True)
        super(News, self).save(*args, **kwargs)

    def __str__(self):
        return str(self.title)



from rest_framework import serializers

from aparnik.api.serializers import ModelSerializer
from aparnik.utils.utils import convert_iran_phone_number_to_world_number

from ..models import ContactUs


class ContactUsListSerializer(ModelSerializer):
    url = serializers.SerializerMethodField()
    resourcetype = serializers.SerializerMethodField()

    def __init__(self, *args, **kwargs):
        super(ContactUsListSerializer, self).__init__(*args, **kwargs)

    class Meta:
        model = ContactUs
        fields = [
            'id',
            'url',
            'first_name',
            'last_name',
            'title',
            'is_active',
            'update_at',
            'resourcetype',
        ]

        read_only_fields = [
            'id',
            'url',
            'is_active',
            'update_at',
            'resourcetype',
        ]

    def get_url(self, obj):
        return self.context['request'].build_absolute_uri(obj.get_api_uri())

    def get_resourcetype(self, obj):
        return 'ContactUs'


class ContactUsDestailSerializer(ContactUsListSerializer):
    phone = serializers.CharField(required=False)

    def __init__(self, *args, **kwargs):
        super(ContactUsDestailSerializer, self).__init__(*args, **kwargs)

    class Meta:
        model = ContactUs
        fields = ContactUsListSerializer.Meta.fields + [
            'website',
            'address',
            'phone',
            'email',
            'content',
        ]

        read_only_fields = ContactUsListSerializer.Meta.read_only_fields + [
        ]

    def validate_phone(self, value):
        return convert_iran_phone_number_to_world_number(value)



# -*- coding: utf-8 -*-


from django.contrib import admin

from dynamic_raw_id.admin import DynamicRawIDMixin
from .models import QA
from aparnik.contrib.reviews.admin import BaseReviewAdmin, BaseReviewInline


# Register your models here.
class QuestionAnswerInline(DynamicRawIDMixin, BaseReviewInline):
    model = QA
    dynamic_raw_id_fields = ['files']
    exclude = []

    def __init__(self, *args, **kwargs):
        Klass = QuestionAnswerInline
        Klass_parent = BaseReviewInline

        super(Klass, self).__init__(*args, **kwargs)
        self.exclude = Klass_parent.exclude + self.exclude

    def get_queryset(self, request):
        return QA.objects.filter(parent_obj__isnull=False)


class QuestionAnswerAdmin(BaseReviewAdmin):

    fields = ['files', 'model_obj', ]
    list_display = []
    search_fields = []
    list_filter = []
    exclude = []
    raw_id_fields = ['model_obj', ]
    dynamic_raw_id_fields = ['files', ]
    inlines = [QuestionAnswerInline]

    def __init__(self, *args, **kwargs):
        Klass = QuestionAnswerAdmin
        Klass_parent = BaseReviewAdmin

        super(Klass, self).__init__(*args, **kwargs)
        self.fields = Klass_parent.fields + self.fields
        self.list_display = Klass_parent.list_display + self.list_display
        self.list_filter = Klass_parent.list_filter + self.list_filter
        self.search_fields = Klass_parent.search_fields + self.search_fields
        self.exclude = Klass_parent.exclude + self.exclude
        self.dynamic_raw_id_fields = Klass_parent.dynamic_raw_id_fields + self.dynamic_raw_id_fields
        self.raw_id_fields = Klass_parent.raw_id_fields + self.raw_id_fields

    class Meta:
        model = QA

    def get_queryset(self, request):
        return QA.objects.filter(parent_obj__isnull=True)


admin.site.register(QA, QuestionAnswerAdmin)



from rest_framework import serializers

from aparnik.packages.shops.products.api.admin.serializers import ProductAdminListSerializer


from aparnik.packages.educations.courses.models import Course


class CourseAdminListSerializer(ProductAdminListSerializer):

    def __init__(self, *args, **kwargs):
        super(CourseAdminListSerializer, self).__init__(*args, **kwargs)

    class Meta:
        model = Course
        fields = ProductAdminListSerializer.Meta.fields + [

        ]

        read_only_fields = ProductAdminListSerializer.Meta.read_only_fields + [

        ]


# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-10-26 16:12


import aparnik.utils.fields
import django.core.validators
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('supports', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='support',
            name='phone',
            field=aparnik.utils.fields.PhoneField(blank=True, max_length=255, null=True, validators=[django.core.validators.RegexValidator(code=b'nomatch', message='phone is not valid, please insert with code', regex=b'^0(?!0)\\d{2}([0-9]{8})$')], verbose_name='Phone'),
        ),
    ]



# Generated by Django 2.2 on 2021-11-29 09:37

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('orders', '0012_auto_20190615_1430'),
    ]

    operations = [
        migrations.AddField(
            model_name='order',
            name='whom',
            field=models.CharField(blank=True, db_index=True, max_length=255, null=True, verbose_name='Whom'),
        ),
    ]



# Generated by Django 2.2 on 2019-08-10 18:36

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('notifications', '0007_auto_20190622_0842'),
    ]

    operations = [
        migrations.AlterField(
            model_name='notification',
            name='title',
            field=models.CharField(max_length=255, verbose_name='Title'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2019-04-28 09:59


from django.db import migrations
import random
import string


def randomPassword():
    """Generate a random password """
    randomSource = string.ascii_letters + string.digits
    password = random.choice(string.ascii_lowercase)
    password += random.choice(string.ascii_uppercase)
    password += random.choice(string.digits)
    for i in range(155):
        password += random.choice(randomSource)
    passwordList = list(password)
    random.SystemRandom().shuffle(passwordList)
    password = ''.join(passwordList)
    return password


def add_keys(apps, schema_editor):
    '''
    We can't import the Post model directly as it may be a newer
    version than this migration expects. We use the historical version.
    '''
    Setting = apps.get_model('settings', 'Setting')
    key = ''
    try:
        key = 'SECRET_KEY'
        Setting.objects.get(key=key)
    except Exception:
        Setting.objects.create(
            title='کلید امنیتی تبادل اطلاعات',
            key=key,
            value=randomPassword(),
            value_type='s',
            is_show=True,
            is_variable_in_home=False,
        )

    try:
        key = 'ORDER_SYNC_WITH_WEBSITES'
        Setting.objects.get(key=key)
    except Exception:
        Setting.objects.create(
            title='ای پی آی سینک شدن فاکتور ها با وبسایت',
            key=key,
            value='',
            value_type='s',
            is_show=False,
            is_variable_in_home=False,
        )


def remove_keys(apps, schema_editor):
    '''
    We can't import the Post model directly as it may be a newer
    version than this migration expects. We use the historical version.
    '''
    Setting = apps.get_model('settings', 'Setting')
    try:
        key = 'SECRET_KEY'
        Setting.objects.get(key=key).delete()
    except Exception:
        pass

    try:
        key = 'ORDER_SYNC_WITH_WEBSITES'
        Setting.objects.get(key=key).delete()
    except Exception:
        pass


class Migration(migrations.Migration):

    dependencies = [
        ('settings', '0016_auto_20190204_1825'),
    ]

    operations = [
        migrations.RunPython(add_keys, reverse_code=remove_keys),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2019-01-20 17:24


import colorfield.fields
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('buttons', '0005_button_title_color'),
    ]

    operations = [
        migrations.AlterField(
            model_name='button',
            name='background_color',
            field=colorfield.fields.ColorField(default='#2672DF', max_length=18, verbose_name='\\u0631\\u0646\\u06af \\u067e\\u0633 \\u0632\\u0645\\u06cc\\u0646\\u0647'),
        ),
        migrations.AlterField(
            model_name='button',
            name='icon_color',
            field=colorfield.fields.ColorField(default='#2672DF', max_length=18, verbose_name='\\u0631\\u0646\\u06af \\u0622\\u06cc\\u06a9\\u0646'),
        ),
        migrations.AlterField(
            model_name='button',
            name='title_color',
            field=colorfield.fields.ColorField(default='#2672DF', max_length=18, verbose_name='Title color'),
        ),
    ]



from rest_framework import serializers
from rest_polymorphic.serializers import PolymorphicSerializer

from aparnik.contrib.basemodels.api.serializers import BaseModelDetailSerializer, BaseModelListSerializer
from aparnik.contrib.segments.api.serializers import BaseSegmentListPolymorphicSerializer
from aparnik.settings import aparnik_settings
from ..models import Teacher

UserSummeryListSerializer = aparnik_settings.USER_SUMMARY_LIST_SERIALIZER


# Teacher List
class TeacherListSerializer(BaseModelListSerializer):
    user_obj = serializers.SerializerMethodField()
    models_api = serializers.SerializerMethodField()

    def __init__(self, *args, **kwargs):
        super(TeacherListSerializer, self).__init__(*args, **kwargs)

    class Meta:
        model = Teacher
        fields = BaseModelListSerializer.Meta.fields + [
            'user_obj',
            'start_date',
            'models_api',
        ]

    def get_user_obj(self, obj):
        return UserSummeryListSerializer(obj.user_obj, many=False, read_only=True, context=self.context).data

    def get_models_api(self, obj):
        return self.context['request'].build_absolute_uri(obj.get_api_models_uri())

# Teacher details
# TODO: use teacher details in base polymorphic serializers
class TeacherDetailsSerializers(TeacherListSerializer, BaseModelDetailSerializer):
    slider_segment = serializers.SerializerMethodField()
    models = serializers.SerializerMethodField()

    def __init__(self, *args, **kwargs):
        super(TeacherListSerializer, self).__init__(*args, **kwargs)

    class Meta:
        model = Teacher
        fields = TeacherListSerializer.Meta.fields + BaseModelDetailSerializer.Meta.fields + [
            'biography',
            'slider_segment',
            'models',
        ]

    def get_slider_segment(self, obj):
        return BaseSegmentListPolymorphicSerializer(obj.slider_segment_obj, many=False, read_only=True, context=self.context).data

    def get_models(self, obj):
        # teacher.course_set.active()
        return 1212


# Teacher List Polymorphic
class TeacherListPolymorphicSerializer(PolymorphicSerializer):

    def __init__(self, *args, **kwargs):
        self.model_serializer_mapping = {
            Teacher: TeacherListSerializer,
        }
        super(TeacherListPolymorphicSerializer, self).__init__(*args, **kwargs)

    model_serializer_mapping = {}


# Teacher Details Polymorphic
class TeacherDetailsPolymorphicSerializer(PolymorphicSerializer):

    def __init__(self, *args, **kwargs):
        self.model_serializer_mapping = {
            Teacher: TeacherDetailsSerializers,
        }
        super(TeacherDetailsPolymorphicSerializer, self).__init__(*args, **kwargs)

    model_serializer_mapping = {}



# Generated by Django 2.2 on 2020-02-10 14:51

import aparnik.contrib.chats.models
from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('filefields', '0013_auto_20190714_1639'),
        ('chats', '0002_chatsession_type'),
    ]

    operations = [
        migrations.AddField(
            model_name='chatsession',
            name='cover_obj',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='filefields.FileField', verbose_name='Cover'),
        ),
        migrations.AddField(
            model_name='chatsession',
            name='title',
            field=models.CharField(blank=True, default='', max_length=255, verbose_name='Title'),
        ),
        migrations.AlterField(
            model_name='chatsession',
            name='owner',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to=settings.AUTH_USER_MODEL, verbose_name='Owner'),
        ),
        migrations.AlterField(
            model_name='chatsession',
            name='uri',
            field=models.URLField(default=aparnik.contrib.chats.models._generate_unique_uri, verbose_name='Uri'),
        ),
    ]





from django.conf.urls import url

from aparnik.contrib.invitation.api.views import InviteDetailAPIView, InviteListAPIView


app_name = 'invitation'

urlpatterns = [
    url(r'^$', InviteListAPIView.as_view(), name='list'),
    url(r'^(?P<id>\d+)/$', InviteDetailAPIView.as_view(), name='detail'),
]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-12-13 16:11


from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('basemodels', '0008_auto_20181211_1552'),
        ('segments', '0005_remove_basesegment_model_obj'),
    ]

    operations = [
        migrations.AddField(
            model_name='basesegment',
            name='model_obj',
            field=models.ManyToManyField(through='segments.SegmentSort', to='basemodels.BaseModel', verbose_name='\\u0686\\u0647 \\u0686\\u06cc\\u0632 \\u0631\\u0627 \\u0646\\u0645\\u0627\\u06cc\\u0634 \\u062f\\u0647\\u062f\\u061f'),
        ),
    ]



from rest_framework import permissions
from rest_framework.generics import get_object_or_404
from rest_framework.permissions import SAFE_METHODS

from aparnik.contrib.settings.models import Setting


class IsAdminPermission(permissions.BasePermission):
    """
    Object-level permission to only allow owners of an object to edit it.
    Assumes the model instance has an `owner` attribute.
    """

    def has_object_permission(self, request, view, obj):
        if request.user and request.user.is_admin:
            return True
        return False


class IsThirdPartyPermission(permissions.BasePermission):
    def has_permission(self, request, view):
        secret_key = get_object_or_404(Setting.objects.active(), key='SECRET_KEY').get_value()
        for method in ['GET', 'POST', 'data']:
            if getattr(request, method, {}).get('secret_key') == secret_key:
                return True
        return False



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2019-05-01 11:42


from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('filefields', '0004_filefield_title'),
    ]

    operations = [
        migrations.AddField(
            model_name='filefield',
            name='is_encrypt_needed',
            field=models.BooleanField(default=False, verbose_name='Is encrypt needed?'),
        ),
        migrations.AddField(
            model_name='filefield',
            name='is_lock',
            field=models.BooleanField(default=False, verbose_name='Is lock?'),
        ),
        migrations.AddField(
            model_name='filefield',
            name='iv',
            field=models.CharField(blank=True, default='', max_length=64, null=True, verbose_name='IV'),
        ),
        migrations.AddField(
            model_name='filefield',
            name='password',
            field=models.CharField(blank=True, default='', max_length=64, null=True, verbose_name='\\u06af\\u0630\\u0631\\u0648\\u0627\\u0698\\u0647'),
        ),
        migrations.AlterField(
            model_name='filefield',
            name='type',
            field=models.CharField(blank=True, choices=[('M', '\\u0641\\u06cc\\u0644\\u0645'), ('V', '\\u0635\\u062f\\u0627'), ('P', '\\u067e\\u06cc \\u062f\\u06cc \\u0627\\u0641'), ('I', '\\u0639\\u06a9\\u0633')], max_length=1, null=True, verbose_name='\\u0646\\u0648\\u0639 \\u0641\\u0627\\u06cc\\u0644'),
        ),
    ]



"""Views for the chat app."""
from django.db.models import F
from django.http import Http404
from rest_framework import permissions, status
from rest_framework.exceptions import ValidationError
from rest_framework.generics import ListAPIView, CreateAPIView, RetrieveUpdateDestroyAPIView, get_object_or_404
from rest_framework.response import Response
from rest_framework.serializers import as_serializer_error
from rest_framework.status import HTTP_400_BAD_REQUEST, HTTP_200_OK
from rest_framework.views import APIView
from rest_framework import filters

from django_filters.rest_framework import DjangoFilterBackend

from .serializers import *
from ..models import ChatSessionTypeEnum


# chat session
class ChatSessionListAPIView(ListAPIView):
    serializer_class = ChatSessionListSerializer
    permission_classes = [permissions.IsAuthenticated]
    filter_backends = (DjangoFilterBackend, filters.SearchFilter)
    filter_fields = ['type', ]

    def get_queryset(self):
        user = self.request.user
        queryset = ChatSession.objects.user_session(user=user).order_by('-message_created_at')
        return queryset


class ChatSessionDetailAPIView(RetrieveUpdateDestroyAPIView):
    serializer_class = ChatSessionDetailsSerializer
    permission_classes = [permissions.IsAuthenticated]
    lookup_url_kwarg = 'uri'
    lookup_field = 'uri'

    def get_queryset(self):
        user = self.request.user
        queryset = ChatSession.objects.user_session(user=user)
        return queryset


class ChatSessionMarkAsReadAPIView(APIView):
    permission_classes = [permissions.IsAuthenticated]

    """
    A view that returns the count of active users in JSON.
    """
    def get(self, request, uri, format=None):

        status = HTTP_400_BAD_REQUEST
        content = {}
        user = request.user

        chat_session = get_object_or_404(ChatSession.objects.all(), uri=uri)

        content['count'] = ChatMessageNotification.objects.mark_as_read_chat_session(user, chat_session)

        status = HTTP_200_OK
        return Response(content, status=status)


# class ChatSessionCreateAPIView(CreateAPIView):
#     serializer_class = ChatSessionDetailsSerializer
#     permission_classes = [permissions.IsAuthenticated]
#
#     def get_queryset(self):
#         return ChatSession.objects.all()
#
#     def perform_create(self, serializer):
#         user = self.request.user
#         serializer.save(owner=user)


class ChatSessionCreateAPIView(APIView):
    """
    Create or fetch a snippet instance.
    """
    permission_classes = [permissions.IsAuthenticated]

    def post(self, request, format=None):
        owner = request.user
        username_mention = request.POST.get('username_mention', None)
        try:
            user = User.objects.find_by_username_mention(username_mention=username_mention)
        except:
            raise Http404
        type = request.POST.get('type', None)
        if type and int(type) == ChatSessionTypeEnum.PRIVATE:
            chat_session = ChatSession.objects.private(owner, user)
            serializer = ChatSessionDetailsSerializer(chat_session, many=False, read_only=True, context={'request': request})
            return Response(serializer.data, status=status.HTTP_200_OK)
        serializer = ChatSessionDetailsSerializer(data=request.data, context={'request': request})
        if serializer.is_valid():
            try:
                serializer.save(owner=owner)
            except Exception as e:
                raise ValidationError(as_serializer_error(e))
            return Response(serializer.data, status=status.HTTP_201_CREATED)
        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)


# chat member
class ChatSessionMemberListAPIView(ListAPIView):
    serializer_class = ChatSessionMemberListSerializer
    permission_classes = [permissions.IsAuthenticated]
    filter_class = []

    def get_queryset(self):
        user = self.request.user
        uri = None
        if 'uri' in self.request.parser_context['kwargs']:
            uri = self.request.parser_context['kwargs']['uri']
        return ChatSessionMember.objects.filter(chat_session__uri=uri)


class ChatSessionMemberDetailAPIView(RetrieveUpdateDestroyAPIView):
    serializer_class = ChatSessionMemberDetailsSerializer
    queryset = ChatSessionMember.objects.all()
    permission_classes = [permissions.IsAuthenticated]
    lookup_url_kwarg = 'id'
    lookup_field = 'id'
    # TODO: check permission for delete id


# message list
class ChatSessionMessageListAPIView(ListAPIView):
    serializer_class = ChatSessionMessageListSerializer
    permission_classes = [permissions.IsAuthenticated]
    filter_class = []

    def get_queryset(self):
        user = self.request.user
        uri = None
        if 'uri' in self.request.parser_context['kwargs']:
            uri = self.request.parser_context['kwargs']['uri']
        return ChatSessionMessage.objects.filter(chat_session__uri=uri).order_by('-created_at')


class ChatSessionMessageDetailAPIView(RetrieveUpdateDestroyAPIView):
    serializer_class = ChatSessionMessageDetailsSerializer
    queryset = ChatSessionMessage.objects.all()
    permission_classes = [permissions.IsAuthenticated]
    lookup_url_kwarg = 'id'
    lookup_field = 'id'
    # TODO: check permission for delete id


class ChatSessionMessageCreateAPIView(CreateAPIView):
    serializer_class = ChatSessionMessageDetailsSerializer
    permission_classes = [permissions.IsAuthenticated]

    def get_queryset(self):
        return ChatSession.objects.all()

    def perform_create(self, serializer):
        user = self.request.user
        chat_session = None
        if 'uri' in self.request.parser_context['kwargs']:
            uri = self.request.parser_context['kwargs']['uri']
            chat_session = get_object_or_404(ChatSession.objects.all(), uri=uri)
        serializer.save(user=user, chat_session=chat_session)

##
# class ChatSessionMessageView(APIView):
#     """Create/Get Chat session messages."""
#
#     permission_classes = (permissions.IsAuthenticated,)
#
#     def get(self, request, *args, **kwargs):
#         """return all messages in a chat session."""
#         uri = kwargs['uri']
#
#         chat_session = ChatSession.objects.get(uri=uri)
#         messages = [chat_session_message.to_json()
#                     for chat_session_message in chat_session.messages.all()]
#
#         return Response({
#             'id': chat_session.id, 'uri': chat_session.uri,
#             'messages': messages
#         })
#
#     def post(self, request, *args, **kwargs):
#         """create a new message in a chat session."""
#         uri = kwargs['uri']
#         message = request.data['message']
#
#         user = request.user
#         chat_session = ChatSession.objects.get(uri=uri)
#
#         chat_session_message = ChatSessionMessage.objects.create(
#             user=user, chat_session=chat_session, message=message
#         )
#
#         notif_args = {
#             'source': user,
#             'source_display_name': user.get_full_name(),
#             'category': 'chat', 'action': 'Sent',
#             'obj': chat_session_message.id,
#             'short_description': 'You a new message', 'silent': True,
#             'extra_data': {
#                 'uri': chat_session.uri,
#                 'message': chat_session_message.to_json()
#             }
#         }
#         notify.send(
#             sender=self.__class__, **notif_args, channels=['websocket']
#         )
#
#         return Response({
#             'status': 'SUCCESS', 'uri': chat_session.uri, 'message': message,
#             'user': deserialize_user(user)
#         })



# Generated by Django 2.2 on 2019-07-01 09:11

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('filefields', '0011_auto_20190615_1430'),
    ]

    operations = [
        migrations.AddField(
            model_name='filefield',
            name='multi_quality',
            field=models.BooleanField(default=False, verbose_name='Multi Quality'),
        ),
        migrations.AddField(
            model_name='filefield',
            name='multi_quality_processing',
            field=models.IntegerField(default=-1, verbose_name='Multi Quality Processing'),
        ),
    ]



from django.conf.urls import url, include
from aparnik.views import install, share

app_name='aparnik'

urlpatterns = [
    url(r'^install$', install, name='install'),
    url(r'^share/', share, name='share'),
    url(r'^shops/', include('aparnik.packages.shops.urls.urls', namespace='shops')),
    url(r'^bank-gateways/', include('aparnik.packages.bankgateways.urls.urls', namespace='bank_gateways')),
]



VERSION = (0, 4)

from .time import *
from .number import *
from .filesize import *
from .i18n import activate, deactivate

__all__ = ['VERSION', 'naturalday', 'naturaltime', 'ordinal', 'intword',
           'naturaldelta', 'intcomma', 'apnumber', 'fractional', 'naturalsize',
           'activate', 'deactivate', 'naturaldate']



# Generated by Django 2.2 on 2020-04-11 12:26

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('aboutus', '0004_auto_20200405_1628'),
    ]

    operations = [
        migrations.AlterField(
            model_name='information',
            name='address',
            field=models.CharField(blank=True, max_length=255, null=True, verbose_name='Address'),
        ),
    ]



"""
Sometimes in your Django model you want to raise a ``ValidationError`` in the ``save`` method, for
some reason.
This exception is not managed by Django Rest Framework because it occurs after its validation
process. So at the end, you'll have a 500.
Correcting this is as simple as overriding the exception handler, by converting the Django
``ValidationError`` to a DRF one.
"""
import logging

from django.core.exceptions import ValidationError as DjangoValidationError
from rest_framework.views import exception_handler as drf_exception_handler
from rest_framework.exceptions import ValidationError

LOG = logging.getLogger(__name__)


def exception_handler(exception, context):
    """Handle Django ValidationError as an accepted exception
    Must be set in settings:
    For the parameters, see ``exception_handler``
    """

    if isinstance(exception, DjangoValidationError):
        if hasattr(exception, "message_dict"):
            detail = exception.message_dict
        elif hasattr(exception, "message"):
            detail = exception.message
        elif hasattr(exception, "messages"):
            detail = exception.messages
        else:
            LOG.error("BAD VALIDATION MESSAGE: %s", exception)
        exception = ValidationError(detail=detail)

    return drf_exception_handler(exception, context)



"""Celery tasks."""

from __future__ import absolute_import, unicode_literals

import json
import time

from celery import shared_task
from django.core.serializers.json import DjangoJSONEncoder
from django.utils.text import Truncator
from fcm_django.models import FCMDevice

from aparnik.contrib.basemodels.api.serializers import ModelListPolymorphicSerializer
from aparnik.contrib.notifications.models import Notification
from aparnik.contrib.settings.models import Setting
from aparnik.contrib.users.api.views import UserSummeryListSerializer
from aparnik.utils.utils import get_request


@shared_task
def send_push_notification(chat_message_id):
    """Send chat message create notification and etc via a channel to celery."""
    time.sleep(0.2)
    notification = Notification.objects.get(pk=chat_message_id)

    icon = 'https://cdn.aparnik.com/static/website/img/logo-persian.png'
    request = get_request()
    try:
        setting = Setting.objects.get(key='LOGO_PROJECT_ICON')
        icon = setting.get_value()
    except:
        pass

    obj = notification

    model_serial = json.dumps(
        ModelListPolymorphicSerializer(
            obj.model_obj,
            many=False,
            read_only=True,
            context={'request': request},
        ).data,
        sort_keys=True,
        indent=1,
        cls=DjangoJSONEncoder
        ) if obj.model_obj else None

    user_serial = json.dumps(
        UserSummeryListSerializer(
            obj.from_user_obj,
            many=False,
            read_only=True,
            context={'request': request},
            ).data,
            sort_keys=True,
            indent=1,
            cls=DjangoJSONEncoder
            ) if obj.from_user_obj else None
    try:
        devices = FCMDevice.objects.filter(user__in=obj.users.all()).order_by('device_id').distinct()
        # TODO: handle response to log and etc...
        sent_result = devices.send_message(
            title=obj.title,
            body=Truncator(obj.description).words(30),
            icon=icon,
            data={
                'model': model_serial,
                'from_user': user_serial,
                'type': obj.type,
                'title': obj.title,
                'description': Truncator(obj.description).words(30),

            }
        )
        obj.sent_result = sent_result
        obj.description_for_admin = None
        obj.update_needed = False
        obj.save()
    except Exception as inst:
        obj.description_for_admin = 'type: {0}\nargs: {1}\ninstance: {2}'.format(type(inst), inst.args, inst)
        obj.save()



# -*- coding: utf-8 -*-


from django.db.models import Count, Q
from django.shortcuts import get_object_or_404
from rest_framework.permissions import (AllowAny, IsAuthenticated)
from rest_framework.generics import RetrieveAPIView, CreateAPIView, UpdateAPIView, ListAPIView

from aparnik.contrib.basemodels.api.views import BaseModelSortAPIView
from .serializers import TicketListSerializer, TicketConversationListSerializer, TicketCreateSerializer, TicketConversationCreateSerializer
from ..models import Ticket, TicketConversation


class TicketSortAPIView(BaseModelSortAPIView):
    permission_classes = [AllowAny]

    """
        A view that returns the count of active users in JSON.
        """
    def __init__(self, *args, **kwargs):
        super(TicketSortAPIView, self).__init__(*args, **kwargs)
        command_dict = {

        }

        self.command_dict.update(command_dict)


class TicketListAPIView(ListAPIView):
    serializer_class = TicketListSerializer
    permission_classes = [IsAuthenticated]

    def get_queryset(self):
        user = self.request.user
        # queryset = Product.objects.active(user=user)
        queryset = None
        if user.is_superuser:
            queryset = Ticket.objects.active(user=user)
        else:
            queryset = Ticket.objects.this_user(user=user)
        sort_api = TicketSortAPIView(request=self.request)
        queryset = sort_api.get_query_sort(queryset)
        return queryset


class TicketCreateAPIView(CreateAPIView):
    serializer_class = TicketCreateSerializer
    permission_classes = [IsAuthenticated]

    def perform_create(self, serializer):
        user = self.request.user
        serializer.save(user_obj=user)


class TicketConversationListAPIView(ListAPIView):
    serializer_class = TicketConversationListSerializer
    permission_classes = [IsAuthenticated]

    def get_queryset(self, *args, **kwargs):
        user = self.request.user
        ticket = get_object_or_404(Ticket, pk=self.request.parser_context['kwargs']['ticket_id'])
        queryset = ticket.ticketconversation_set.all()
        return queryset


class TicketConversationCreateAPIView(CreateAPIView):
    serializer_class = TicketConversationCreateSerializer
    permission_classes = [IsAuthenticated]

    def perform_create(self, serializer, *args, **kwargs):
        ticket = get_object_or_404(Ticket, pk=self.request.parser_context['kwargs']['ticket_id'])
        user = self.request.user
        files = self.request.data.get('files', None)
        if files:
            files = files.split(',')
        dict = {
            'user_obj': user,
            'files_obj': [],
            'ticket_obj': ticket,
        }
        if files:
            dict['files_obj'] = files
        serializer.save(**dict)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2019-03-15 13:47


from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('notifiesme', '0001_initial'),
        ('sliders', '0004_sliderimage'),
        ('counters', '0002_auto_20190115_1425'),
        ('segments', '0012_auto_20181214_1330'),
        ('notifications', '0001_initial'),
        ('reviews', '0004_reviewsummary_percentage'),
        ('bookmarks', '0002_auto_20181026_1745'),
        ('buttons', '0006_auto_20190120_1724'),
        ('cosales', '0004_auto_20190315_1340'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='cosalehistory',
            name='basemodel_ptr',
        ),
        migrations.RemoveField(
            model_name='cosalehistory',
            name='cosale_obj',
        ),
        migrations.RemoveField(
            model_name='cosale',
            name='status',
        ),
        migrations.DeleteModel(
            name='CoSaleHistory',
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2019-01-21 21:01


from django.db import migrations, models
import uuid


class Migration(migrations.Migration):

    dependencies = [
        ('orders', '0007_auto_20190121_1856'),
    ]

    operations = [
        migrations.AlterField(
            model_name='order',
            name='uuid',
            field=models.UUIDField(default=uuid.uuid4, editable=False, unique=True, verbose_name='Reference ID'),
        ),
    ]



from django.utils.translation import ugettext_lazy as _
from django.shortcuts import get_object_or_404
from django.contrib.auth import get_user_model

from rest_framework.generics import RetrieveAPIView, ListAPIView
from rest_framework.permissions import (IsAuthenticated, AllowAny)
from rest_framework.response import Response
from rest_framework.status import HTTP_200_OK, HTTP_400_BAD_REQUEST
from rest_framework.views import APIView
from rest_framework.serializers import as_serializer_error
from rest_framework.exceptions import ValidationError

from aparnik.settings import Setting
from aparnik.utils.utils import convert_iran_phone_number_to_world_number
from aparnik.contrib.addresses.models import UserAddress
from aparnik.packages.shops.products.models import Product
from aparnik.contrib.basemodels.api.serializers import ModelListPolymorphicSerializer, ModelDetailsPolymorphicSerializer
from ..models import Order

User = get_user_model()


class OrderListAPIView(ListAPIView):
    serializer_class = ModelDetailsPolymorphicSerializer
    permission_classes = [IsAuthenticated]

    # filter_backends = (filters.SearchFilter,)

    def get_queryset(self):
        user = self.request.user
        if user.is_authenticated:
            return Order.objects.get_this_user(user=user)

        return Order.objects.all()


class OrderDetailAPIView(RetrieveAPIView):
    serializer_class = ModelDetailsPolymorphicSerializer
    permission_classes = [IsAuthenticated]
    lookup_url_kwarg = 'id'
    lookup_field = 'id'

    def get_queryset(self):
        user = self.request.user

        return Order.objects.get_this_user(user=user)


class AddProductOrderAPIView(APIView):
    permission_classes = [IsAuthenticated]

    def post(self, request, id=None, format=None, *args, **kwargs):
        status = HTTP_400_BAD_REQUEST
        user = self.request.user
        # id = product_id
        content = {
            'success': False,
            'msg': '',
            'data': {
            },
        }
        order = None
        products = None
        products_id = request.POST.get('products_id').split(',')
        quantity = request.POST.get('quantity', 1)
        try:
            products = Product.objects.filter(id__in=products_id)
            if id:
                order = get_object_or_404(Order.objects.all(), id=id)
            else:
                order = Order.objects.create(user=user)
        except Exception as e:
            raise ValidationError(as_serializer_error(e))

        for product in products.all():
            order.add_item(product=product, quantity=quantity)

        order.save()

        content['data']['order'] = ModelDetailsPolymorphicSerializer(order, many=False, read_only=True,
                                                          context={'request': self.request}).data
        content['success'] = True
        status = HTTP_200_OK
        return Response(content, status=status)


class AddProductOrderByWebsitesAPIView(APIView):
    permission_classes = [AllowAny]

    def post(self, request, id=None, format=None, *args, **kwargs):
        status = HTTP_400_BAD_REQUEST
        content = {

        }
        secret_key_post = request.POST.get('secret_key', None)
        secret_key = get_object_or_404(Setting.objects.active(), key='SECRET_KEY').get_value()
        if secret_key != secret_key_post:
            raise ValidationError({
                'non_fields_error': _("Bad Request.")
            })
        username = request.POST.get('username', None)
        if username:
            username = convert_iran_phone_number_to_world_number(username)
        try:
            user = User.objects.get(username=username)
        except:
            user = User.objects.create(username=username)

        self.request.user = user

        order = None
        products = None
        products_id = request.POST.get('products_id').split(',')
        whom = request.POST.get('whom')

        if whom == None and request.path == '/api/v1/aparnik/shops/orders/websites/pay/1.1/':
            raise ValidationError({
                'whom': _('This field is required.')
            })

        quantity = request.POST.get('quantity', 1)

        try:
            products = Product.objects.filter(id__in=products_id)
            if id:
                order = get_object_or_404(Order.objects.all(), id=id)
            else:
                order = Order.objects.create(user=user, whom=whom)
        except Exception as e:
            raise ValidationError(as_serializer_error(e))

        for product in products.all():
            order.add_item(product=product, quantity=quantity)

        order.is_sync_with_websites = True
        order.save()
        order.pay_success(status=Order.STATUS_PAID_BY_WEBSITE)
        content = ModelDetailsPolymorphicSerializer(order, many=False, read_only=True,
                                                                     context={'request': self.request}).data
        status = HTTP_200_OK
        return Response(content, status=status)


class RemoveProductOrderAPIView(APIView):
    permission_classes = [IsAuthenticated]

    def get(self, request, id, item_id, format=None, *args, **kwargs):
        status = HTTP_400_BAD_REQUEST
        user = self.request.user
        # id = product_id
        content = {

        }
        order = None

        try:
            from ..models import OrderItem
            order = get_object_or_404(Order.objects.all(), id=id)
            item = get_object_or_404(OrderItem.objects.all(), id=item_id)
            if order.is_success:
                raise ValidationError({
                    'order': _("This order finish before.")
                })
            item.delete()
        except Exception as e:
            raise ValidationError(as_serializer_error(e))

        order.save()

        content = ModelDetailsPolymorphicSerializer(order, many=False, read_only=True,
                                                          context={'request': self.request}).data
        status = HTTP_200_OK
        return Response(content, status=status)


class ChargeWalletAPIView(APIView):
    permission_classes = [IsAuthenticated]

    def post(self, request, format=None, *args, **kwargs):
        status = HTTP_400_BAD_REQUEST
        user = self.request.user
        # id = product_id
        content = {
            'success': False,
            'msg': '',
            'data': {
            },
        }
        order = None
        product = Product.objects.get_wallet()
        count = request.POST.get('toman', 10)
        try:
            order = Order.objects.create(user=user)
        except Exception as e:
            raise ValidationError(as_serializer_error(e))

        order.add_item(product=product, quantity=count)

        order.save()

        content['data']['order'] = ModelDetailsPolymorphicSerializer(order, many=False, read_only=True,
                                                          context={'request': self.request}).data
        content['success'] = True
        status = HTTP_200_OK
        return Response(content, status=status)


class AddOrderAddressAPIView(APIView):
    permission_classes = [IsAuthenticated]

    def post(self, request, id):
        user = self.request.user
        address_id = self.request.data.get('address_id', None)

        user_address = get_object_or_404(UserAddress.objects.active(), id=address_id, user_obj=user)
        order = get_object_or_404(Order.objects.active(), id=id, user=user)

        try:
            order.address_obj = user_address
            order.save()

            content = ModelDetailsPolymorphicSerializer(order,
                                                     many=False, read_only=True,
                                                     context={'request': self.request}).data
            status = HTTP_200_OK
            return Response(content, status=status)

        except Exception as e:
            raise ValidationError(as_serializer_error(e))



# Generated by Django 2.2 on 2021-01-03 13:03

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('azbankgateways', '0002_auto_20210102_0721'),
        ('payments', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='PaymentBank',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='Created at')),
                ('update_at', models.DateTimeField(auto_now=True, verbose_name='Update at')),
                ('bank_record', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='azbankgateways.Bank', verbose_name='Bank record')),
                ('payment', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='payments.Payment', verbose_name='Payment')),
            ],
            options={
                'verbose_name': 'Payment Bank',
                'verbose_name_plural': 'Payments Bank',
                'unique_together': {('bank_record',)},
            },
        ),
    ]



from django.db.models import Count, Q, F


from aparnik.contrib.basemodels.models import BaseModel
from aparnik.contrib.bookmarks.models import Bookmark
from aparnik.contrib.suit.api.views import AdminListAPIView, ModelAdminSortAPIView
from aparnik.packages.shops.orders.models import Order
from .serializers import BaseModel, BaseModelAdminSerializer


class BaseModelSortAdminAPIView(ModelAdminSortAPIView):

    """
        A view that returns the count of active users in JSON.
        """
    def __init__(self, *args, **kwargs):
        super(BaseModelSortAdminAPIView, self).__init__(*args, **kwargs)
        command_dict = {

        }
        self.command_dict.update(BaseModel.sort_visit())
        self.command_dict.update(Bookmark.sort_bookmark(prefix='bookmark_obj'))
        self.command_dict.update(command_dict)


class BaseModelAdminListAPIView(AdminListAPIView):
    serializer_class = BaseModelAdminSerializer
    queryset = BaseModel.objects.all()

    def get_sort_api(self, request):
        return BaseModelSortAdminAPIView(request=self.request)



# -*- coding: utf-8 -*-
from rest_framework import serializers

from aparnik.api.serializers import ModelSerializer
from aparnik.contrib.users.api.serializers import UserSummaryListSerializer
from aparnik.contrib.filefields.api.serializers import FileFieldListSerailizer
from aparnik.contrib.basemodels.api.serializers import BaseModelDetailSerializer
from ..models import Ticket, TicketConversation


class TicketListSerializer(BaseModelDetailSerializer):

    user = serializers.SerializerMethodField()
    url_conversation = serializers.SerializerMethodField()
    url_add_conversation = serializers.SerializerMethodField()

    def __init__(self, *args, **kwargs):
        super(TicketListSerializer, self).__init__(*args, **kwargs)

    class Meta:
        model = Ticket
        fields = BaseModelDetailSerializer.Meta.fields + [
            'url_conversation',
            'url_add_conversation',
            'user',
            'title',
            'priority',
            'status',
            'uuid',
            'department',
            'created_at'
        ]

        read_only_fields = BaseModelDetailSerializer.Meta.read_only_fields + [
            'url_conversation',
            'url_add_conversation',
            'user',
        ]

    def get_url_conversation(self, obj):
        return self.context['request'].build_absolute_uri(obj.get_api_conversation_uri())

    def get_url_add_conversation(self, obj):
        return self.context['request'].build_absolute_uri(obj.get_api_add_conversation_uri())

    def get_user(self, obj):
        return UserSummaryListSerializer(obj.user_obj, many=False, read_only=True, context=self.context).data


class TicketCreateSerializer(TicketListSerializer):
    # model_obj_id = serializers.IntegerField(write_only=True, required=True)

    class Meta:
        model = Ticket
        fields = TicketListSerializer.Meta.fields + [
            'resourcetype',
        ]
        read_only_fields = TicketListSerializer.Meta.read_only_fields + [
            'resourcetype'
        ]


class TicketConversationListSerializer(ModelSerializer):

    user = serializers.SerializerMethodField()
    files = serializers.SerializerMethodField()
    resourcetype = serializers.SerializerMethodField()

    def __init__(self, *args, **kwargs):
        super(TicketConversationListSerializer, self).__init__(*args, **kwargs)

    class Meta:
        model = TicketConversation
        fields = [
            'id',
            'user',
            'content',
            'files',
            'created_at',
            'resourcetype',
        ]

        read_only_fields = [
            'user',
            'files',
            'resourcetype',
        ]

    def get_user(self, obj):
        return UserSummaryListSerializer(obj.user_obj, many=False, read_only=True, context=self.context).data


    def get_files(self, obj):
        return FileFieldListSerailizer(obj.files_obj, many=True, read_only=True, context=self.context).data

    def get_resourcetype(self, obj):
        return 'TicketConversation'


class TicketConversationCreateSerializer(TicketConversationListSerializer):

    class Meta:
        model = TicketConversation
        fields = TicketConversationListSerializer.Meta.fields + [

        ]
        read_only_fields = TicketConversationListSerializer.Meta.read_only_fields + [

        ]



# -*- coding: utf-8 -*-


from django.contrib import admin
from jalali_date.admin import ModelAdminJalaliMixin

from aparnik.contrib.users.admin import get_update_at
from .models import Bank
# Register your models here.


def get_payment(obj):
    return obj.payment.uuid


class BankAdmin(admin.ModelAdmin):
    list_display = ('status', 'authority_id', 'ref_id', get_payment, get_update_at)
    # list_display_links = ('username', 'first_name', 'last_name')
    # list_filter = ('is_staff', )
    # list_editable = ('username', 'mobile', )
    # search_fields = ('username', 'last_name', 'first_name')
    # readonly_fields = ( )
    # inlines = [DocumentFilesInline, ]

    actions = None

    class Meta:
        model = Bank

    # permission
    def has_delete_permission(self, request, obj=None):
        # Disable delete
        return False

    def has_add_permission(self, request):
        return True

    def change_view(self, request, object_id, form_url='', extra_context=None):
        ''' customize edit form '''
        extra_context = extra_context or {}
        extra_context['show_save_and_continue'] = False
        extra_context['show_save'] = False
        extra_context['show_save_and_add_another'] = False  # this not works if has_add_permision is True
        return super(BankAdmin, self).change_view(request, object_id, extra_context=extra_context)

admin.site.register(Bank, BankAdmin)



# -*- coding: utf-8 -*-


from django.contrib import admin
from aparnik.contrib.users.admin import get_update_at, get_user_search_fields
from aparnik.contrib.basemodels.admin import BaseModelAdmin, BaseModelTabularInline

from .models import ChatSession, ChatSessionMember, ChatSessionMessage


# Register your models here.
# Inline exercise
class ChatSessionMessageInline(BaseModelTabularInline):
    model = ChatSessionMessage
    raw_id_fields = ['user', 'file_obj']
    fk_name = 'chat_session'
    exclude = []
    extra = 1

    def __init__(self, *args, **kwargs):
        Klass = ChatSessionMessageInline
        Klass_parent = BaseModelTabularInline

        super(Klass, self).__init__(*args, **kwargs)
        self.exclude = Klass_parent.exclude + self.exclude


class ChatSessionMemberInline(BaseModelTabularInline):
    model = ChatSessionMember
    raw_id_fields = ['user']
    fk_name = 'chat_session'
    exclude = []
    extra = 1

    def __init__(self, *args, **kwargs):
        Klass = ChatSessionMemberInline
        Klass_parent = BaseModelTabularInline

        super(Klass, self).__init__(*args, **kwargs)
        self.exclude = Klass_parent.exclude + self.exclude


class ChatSessionAdmin(BaseModelAdmin):
    fields = ['owner', 'cover_obj', 'title', ]
    list_display = ['uri', get_update_at,]
    inlines = [ChatSessionMemberInline, ChatSessionMessageInline]
    list_filter = []
    search_fields = get_user_search_fields('owner') + []
    exclude = []
    raw_id_fields = ['owner', 'cover_obj', ]
    dynamic_raw_id_fields = []

    def __init__(self, *args, **kwargs):
        Klass = ChatSessionAdmin
        Klass_parent = BaseModelAdmin

        super(Klass, self).__init__(*args, **kwargs)
        self.fields = Klass_parent.fields + self.fields
        self.list_display = Klass_parent.list_display + self.list_display
        self.list_filter = Klass_parent.list_filter + self.list_filter
        self.search_fields = Klass_parent.search_fields + self.search_fields
        self.exclude = Klass_parent.exclude + self.exclude
        self.dynamic_raw_id_fields = Klass_parent.dynamic_raw_id_fields + self.dynamic_raw_id_fields
        self.raw_id_fields = Klass_parent.raw_id_fields + self.raw_id_fields




admin.site.register(ChatSession, ChatSessionAdmin)



# -*- coding: utf-8 -*-


from django.shortcuts import render, get_object_or_404

from aparnik.settings import Setting
from aparnik.utils.formattings import formatprice
from aparnik.packages.shops.orders.models import Order
from aparnik.packages.shops.payments.models import Payment
from aparnik.contrib.province.models import City


# Create your views here.
def invoice(request, uuid):

    order = get_object_or_404(Order, uuid=uuid)
    if order.is_success:
        url = None
    else:
        payment = Payment.objects.request_pay(order=order, method=Payment.METHOD_BANK, call_back_url=request.build_absolute_uri(order.get_pay_uri()), user=order.user)
        url = request.build_absolute_uri(payment.get_url())

    logo = Setting.objects.get(key='LOGO_PROJECT_ICON').get_value()
    mohr = Setting.objects.get(key='MOHR_ICON').get_value()
    project_name = Setting.objects.get(key='PROJECT_NAME').get_value()
    economical_number = Setting.objects.get(key='ECONOMICAL_NUMBER').get_value()
    registration_number = Setting.objects.get(key='REGISTRATION_NUMBER').get_value()
    postal_code = Setting.objects.get(key='POSTAL_CODE').get_value()
    city = City.objects.get(id=Setting.objects.get(key='PROJECT_CITY_ID').get_value())
    address = Setting.objects.get(key='ADDRESS').get_value()
    phone = Setting.objects.get(key='PHONE').get_value()
    account_bank_number = Setting.objects.get(key='ACCOUNT_BANK_NUMBER').get_value()
    # چون هر روز ممکنه تغییر کنه این کار منطقی نیست
    dollar_price = '%s' % formatprice.format_price(Setting.objects.get(key='DOLLAR_TO_IRR').get_value() / 10)

    context = {
        'order': order,
        'url_pay': url,
        'customer_address': order.address_obj,
        'logo_small': logo,
        'mohr': mohr,
        'company_name': project_name,
        'economical_number': economical_number,
        'registration_number': registration_number,
        'postal_code': postal_code,
        'city': city,
        'address': address,
        'phone': phone,
        'dollar_price': dollar_price,
        'account_bank_number': account_bank_number,
    }
    template_name = 'orders/invoice.html'
    return render(request, template_name=template_name, context=context)



# Generated by Django 2.2 on 2019-07-01 18:44

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('aparnik_users', '0028_auto_20190615_1430'),
    ]

    operations = [
        migrations.CreateModel(
            name='UserTokenManager',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
            ],
        ),
        migrations.CreateModel(
            name='UserToken',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('token', models.CharField(max_length=500, verbose_name='Token')),
                ('is_active', models.BooleanField(default=True, verbose_name='Is active')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('update_at', models.DateTimeField(auto_now=True)),
                ('device_obj', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='aparnik_users.DeviceLogin', verbose_name='Device')),
                ('user_obj', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='token_user', to=settings.AUTH_USER_MODEL, verbose_name='User')),
            ],
            options={
                'unique_together': {('token', 'user_obj')},
            },
        ),
    ]



# -*- coding: utf-8 -*-


from django.conf import settings

from rest_framework.generics import RetrieveAPIView, CreateAPIView, UpdateAPIView, ListAPIView
from rest_framework.mixins import UpdateModelMixin
from rest_framework.permissions import (AllowAny, IsAuthenticated, IsAdminUser)
from rest_framework.views import APIView
from rest_framework.response import Response
from rest_framework.status import HTTP_200_OK, HTTP_400_BAD_REQUEST

from s3direct.utils import get_s3direct_destinations
import os, json, boto3

from .serializers import FileFieldDetailsSerailizer, FileFieldListSerailizer
from ..models import FileField


class FileFieldListAPIView(ListAPIView):
    serializer_class = FileFieldListSerailizer
    permission_classes = [IsAdminUser]
    queryset = FileField.objects.active()


class SignS3(APIView):
    permission_classes = [IsAuthenticated]

    def post(self, request, format=None, *args, **kwargs):
        # Load necessary information into the application
        S3_BUCKET = settings.AWS_STORAGE_BUCKET_NAME

        # Load required data from the request
        file_name = request.POST.get('file-name')
        file_type = request.POST.get('file-type')

        # Initialise the S3 client
        s3 = boto3.client('s3')
        key = get_s3direct_destinations().get('file').get('key')(file_name)
        endpoint = 'https://' + settings.AWS_S3_CUSTOM_DOMAIN + '/' + key

        # Generate and return the presigned URL
        presigned_post = s3.generate_presigned_post(
            Bucket=S3_BUCKET,
            # Key='upload_file/' + file_name,
            Key=key,
            Fields={"acl": "public-read", "Content-Type": file_type},
            Conditions=[
              {"acl": "public-read"},
              {"Content-Type": file_type}
            ],
            ExpiresIn=3600
        )

        # Return the data to the client
        status = HTTP_200_OK
        content = {
            'data': presigned_post,
            'url': 'https://%s.s3.amazonaws.com/%s' % (S3_BUCKET, key),
            'endpoint': endpoint,
        }

        return Response(content, status=status)


class FileFieldCreateAPIView(CreateAPIView):
    serializer_class = FileFieldDetailsSerailizer
    permission_classes = [AllowAny]




# -*- coding: utf-8 -*-


from rest_framework import serializers

from aparnik.contrib.invitation.models import Invite
from aparnik.settings import aparnik_settings

UserSummaryListSerializer = aparnik_settings.USER_SUMMARY_LIST_SERIALIZER

# Invite
class InviteListSerializer(serializers.ModelSerializer):
    resourcetype = serializers.SerializerMethodField()
    invite = serializers.SerializerMethodField()
    # invited_by = serializers.SerializerMethodField()

    class Meta:
        model = Invite
        fields = (
            'invite',
            # 'invited_by',
            'resourcetype',
            'update_at',

        )

        read_only_fields = [
            'resourcetype',
        ]

    def get_invite(self, obj):
        return UserSummaryListSerializer(obj.invite, read_only=False, context=self.context).data

    def get_resourcetype(self, obj):
        return 'Invite'

    # def get_invited_by(self, obj):
    #     return UserDetailSerializer(obj.invited_by, read_only=False, context=self.context).data

class InviteDetailSerializer(serializers.ModelSerializer):

    invite = serializers.SerializerMethodField()
    # invited_by = serializers.SerializerMethodField()

    class Meta:
        model = Invite
        fields = (
            'invite',
            # 'invited_by',
            'update_at',
        )

        def get_invite(self, obj):
            return UserDetailSerializer(obj.invite, read_only=False, context=self.context).data

        # def get_invited_by(self, obj):
        #     return UserDetailSerializer(obj.invited_by, read_only=False, context=self.context).data



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2019-02-04 18:25


from django.db import migrations


def add_keys(apps, schema_editor):
    '''
    We can't import the Post model directly as it may be a newer
    version than this migration expects. We use the historical version.
    '''
    from django.apps import apps
    Setting = apps.get_model('settings', 'Setting')
    FileField = apps.get_model('filefields', 'FileField')

    key = ''
    for key_file_type, value_file_type in FileField.FILE_TYPE:
        try:
            value = ''
            if key_file_type == 'P':
                value = 'https://cdn.aparnik.com/static/img/icon_pdf.png'
            elif key_file_type == 'M':
                value = 'https://cdn.aparnik.com/static/img/icon_movie.png'
            elif key_file_type == 'V':
                value = 'https://cdn.aparnik.com/static/img/icon_voice.png'
            elif key_file_type == 'I':
                value = 'https://cdn.aparnik.com/static/img/icon_image.png'
            elif key_file_type == 'L':
                value = 'https://cdn.aparnik.com/static/img/icon_image.png'
            else:
                continue

            key = 'FILE_TYPE_%s_ICON' % key_file_type
            Setting.objects.get(key=key)
        except Exception:
            Setting.objects.create(
                title='آیکن مربوط به فایل تایپ %s'% value_file_type,
                key=key,
                value=value,
                value_type='s',
                is_show=False,
                is_variable_in_home=False,
            )


class Migration(migrations.Migration):

    dependencies = [
        ('settings', '0015_auto_20190201_1930'),
    ]

    operations = [
        migrations.RunPython(add_keys, reverse_code=migrations.RunPython.noop),
    ]



from django.conf.urls import url, include

app_name='aparnik'

urlpatterns = [
    url(r'^audits/', include('aparnik.contrib.audits.api.urls', namespace='audits')),
    url(r'^users/', include('aparnik.contrib.users.api.urls', namespace='users')),
    url(r'^addresses/', include('aparnik.contrib.addresses.api.urls', namespace='addresses')),
    url(r'^bankaccounts/', include('aparnik.contrib.bankaccounts.api.urls', namespace='bankaccounts')),
    url(r'^aboutus/', include('aparnik.contrib.aboutus.api.urls', namespace='aboutus')),
    url(r'^contactus/', include('aparnik.contrib.contactus.api.urls', namespace='contactus')),
    url(r'^invitations/', include('aparnik.contrib.invitation.api.urls', namespace='invitations')),
    url(r'^filefields/', include('aparnik.contrib.filefields.api.urls', namespace='files')),
    url(r'^provinces/', include('aparnik.contrib.province.api.urls', namespace='provinces')),
    url(r'^models/', include('aparnik.contrib.basemodels.api.urls', namespace='models')),
    url(r'^bookmarks/', include('aparnik.contrib.bookmarks.api.urls', namespace='bookmarks')),
    url(r'^reviews/', include('aparnik.contrib.reviews.api.urls', namespace='reviews')),
    url(r'^qa/', include('aparnik.contrib.questionanswers.api.urls', namespace='qa')),
    url(r'^pages/', include('aparnik.contrib.pages.api.urls', namespace='pages')),
    url(r'^sliders/', include('aparnik.contrib.sliders.api.urls', namespace='sliders')),
    url(r'^notifications/', include('aparnik.contrib.notifications.api.urls', namespace='notifications')),
    url(r'^notifiesme/', include('aparnik.contrib.notifiesme.api.urls', namespace='notifiesme')),
    url(r'^supports/', include('aparnik.contrib.supports.api.urls', namespace='supports')),
    url(r'^faq/', include('aparnik.contrib.faq.api.urls', namespace='faq')),
    url(r'^tickets/', include('aparnik.contrib.tickets.api.urls', namespace='tickets')),
    url(r'^termsandconditions/', include('aparnik.contrib.termsandconditions.api.urls', namespace='termsandconditions')),
    url(r'^categories/', include('aparnik.contrib.categories.api.urls', namespace='categories')),
    url(r'^chats/', include('aparnik.contrib.chats.api.urls', namespace='chats')),
    url(r'^segments/', include('aparnik.contrib.segments.api.urls', namespace='segments')),
    url(r'^shops/', include('aparnik.packages.shops.urls.api', namespace='shops')),
    url(r'^bank-gateways/', include('aparnik.packages.bankgateways.urls.api', namespace='bank_gateways')),
    url(r'^educations/', include('aparnik.packages.educations.urls.api', namespace='educations')),
    url(r'^news/', include('aparnik.packages.news.api.urls', namespace='news')),
]



# -*- coding: utf-8 -*-


from django.apps import AppConfig
from django.utils.translation import ugettext_lazy as _


class UsersConfig(AppConfig):
    name = 'aparnik.contrib.users'
    label = 'aparnik_users'
    verbose_name = _('Users')



from django.conf.urls import url

from aparnik.contrib.province.api.views import ShahrakCreateAPIView, ShahrakDetailAPIView, ShahrakListAPIView

app_name = 'province'

urlpatterns = [
    url(r'^$', ShahrakListAPIView.as_view(), name='list'),
    url(r'^create/$', ShahrakCreateAPIView.as_view(), name='create'),
    url(r'^(?P<id>\d+)/$', ShahrakDetailAPIView.as_view(), name='detail')
]


# -*- coding: utf-8 -*-


from django.contrib import admin

from aparnik.contrib.basemodels.admin import BaseModelAdmin
from aparnik.settings import aparnik_settings

from .models import FileField


class FileFieldAdmin(BaseModelAdmin):

    fields = ['title', 'is_encrypt_needed', 'multi_quality', 'file_external_url']
    list_display = ['title', 'type', 'is_lock', 'is_encrypt_needed', 'seconds', 'file_size_readable', 'multi_quality', 'multi_quality_processing']
    list_filter = ['type', 'is_lock', 'is_encrypt_needed', 'multi_quality', 'multi_quality_processing']
    search_fields = ['title', 'type']
    exclude = []
    raw_id_fields = []
    dynamic_raw_id_fields = []

    readonly_fields = ()

    def __init__(self, *args, **kwargs):
        Klass = FileFieldAdmin
        Klass_parent = BaseModelAdmin

        super(Klass, self).__init__(*args, **kwargs)
        self.fields = Klass_parent.fields + self.fields
        self.list_display = Klass_parent.list_display + self.list_display
        self.list_filter = Klass_parent.list_filter + self.list_filter
        self.search_fields = Klass_parent.search_fields + self.search_fields
        self.exclude = Klass_parent.exclude + self.exclude
        self.dynamic_raw_id_fields = Klass_parent.dynamic_raw_id_fields + self.dynamic_raw_id_fields
        self.raw_id_fields = Klass_parent.raw_id_fields + self.raw_id_fields

        if aparnik_settings.AWS_ACTIVE:
            file_field_name = ['file_s3', 'size', 'type', 'password', 'iv']
        else:
            file_field_name = ['file_direct']

        self.fields = self.fields + file_field_name
        self.list_display = self.list_display + file_field_name

    class Meta:
        model = FileField


admin.site.register(FileField, FileFieldAdmin)


"""Messaging model."""

from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.conf import settings
from django.contrib.auth import get_user_model

from .fields import JSONField, ListField


class MessagingQuerySet(models.QuerySet):
    """Messagings QuerySet."""

    def all_unread(self):
        """Return all unread notifications."""

        return self.filter(is_read=False)

    def all_read(self):
        """Return all read notifications."""

        return self.filter(is_read=True)


class Messaging(models.Model):
    """
    Model for notifications.

    Parameters:
    ----------
    source: A ForeignKey to Django's User model
        (Can be null if it's not a User to User Messaging)

    source_display_name: A User Friendly name
        for the source of the notification.

    recipient: The Recipient of the notification.
        It's a ForeignKey to Django's User model.

    action: Verbal action for the notification
        E.g Sent, Cancelled, Bought e.t.c

    obj: The id of the object associated with the notification
        (Can be null)

    short_description: The body of the notification.

    url: The url of the object associated with the notification
        (Can be null)
    channels: Channel(s) that were/was used to deliver the message

    extra_data: Extra information that was passed in the notification
        (Optional but default value is an empty dict {})
    """

    User = settings.AUTH_USER_MODEL

    class Meta:
        """Specify ordering for notifications."""
        ordering = ('-id',)

    source = models.ForeignKey(User, null=True, on_delete=models.CASCADE)
    source_display_name = models.CharField(max_length=150, null=True)
    recipient = models.ForeignKey(
        User, related_name='notifications', null=True, 
        on_delete=models.CASCADE
    )
    action = models.CharField(max_length=50)
    category = models.CharField(max_length=50)
    obj = models.IntegerField(null=True, blank=True)
    uri = models.CharField(max_length=255, null=True, verbose_name=_('URI'))
    url = models.URLField(null=True, blank=True)
    short_description = models.CharField(max_length=100)
    channels = ListField(max_length=200)
    extra_data = JSONField(default={})
    is_read = models.BooleanField(default=False)

    create_date = models.DateTimeField(auto_now_add=True)
    update_date = models.DateTimeField(auto_now=True)

    # register queryset
    objects = MessagingQuerySet.as_manager()

    def __str__(self):
        if self.source:
            res = '{}: {} {} {} => {}'.format(
                self.category, self.source, self.action,
                self.short_description, self.recipient
            )
        else:
            res = self.short_description

        return res

    def read(self):
        """Mark notification as read."""
        self.is_read = True
        self.save()



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-12-03 15:33


from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('vouchers', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='voucher',
            name='order_item_obj',
            field=models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='voucher_order_item', to='orders.OrderItem', verbose_name='Order Item'),
        ),
    ]



"""Celery tasks."""

from __future__ import absolute_import, unicode_literals

import time

from celery import shared_task

from aparnik.contrib.chats.api.serializers import ChatSessionMessageDetailsSerializer
from aparnik.contrib.chats.models import *
from aparnik.contrib.messaging.utils import notify
from aparnik.utils.utils import get_request


@shared_task
def send_chat_message(chat_message_id):
    """Send chat message create notification and etc via a channel to celery."""
    time.sleep(0.2)
    chat_message = ChatSessionMessage.objects.get(pk=chat_message_id)
    chat_session = chat_message.chat_session
    user = chat_message.user
    owner = chat_session.owner

    """Send notification via a channel to celery."""
    serializer = ChatSessionMessageDetailsSerializer(chat_message, many=False, read_only=True,
                                                     context={'request': get_request()})
    notify(
        uri=chat_session.uri,
        source=user,
        source_display_name=user.get_full_name(),
        recipient=None,
        action='Create',
        obj=chat_message,
        short_description='You a new message',
        extra_data={
            'uri': chat_session.uri,
            'message': serializer.data,
        },
        channels=['websocket'],
        silent=True,
    )

    """send push notification"""

    def chat_append_to_list(instance, list):
        list.append(instance)
        notify(
            uri=chat_session.uri,
            source=user,
            source_display_name=user.get_full_name(),
            recipient=member.user,
            action='Create',
            obj=chat_message,
            short_description='You a new message',
            extra_data={
                'uri': chat_session.uri,
                'message': serializer.data,
            },
            channels=['push'],
            silent=True,
        )
        return list

    # Best practise send push notification to topics but django fcm didnt support yet.
    chat_notifications = []
    members = chat_session.members.prefetch_related('user').exclude(user=user)
    for member in members:
        chat_notifications = chat_append_to_list(ChatMessageNotification(
            chat_message=chat_message,
            user=member.user,
        ), chat_notifications, )

    if owner.pk != user.pk:
        chat_notifications = chat_append_to_list(ChatMessageNotification(
            chat_message=chat_message,
            user=owner,
        ), chat_notifications, )

    ChatMessageNotification.objects.bulk_create(chat_notifications)



# Generated by Django 2.2 on 2019-06-15 14:30

import aparnik.utils.utils
from django.db import migrations, models
import s3direct.fields


class Migration(migrations.Migration):

    dependencies = [
        ('filefields', '0010_filefield_is_decrypt_needed'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='filefield',
            options={'verbose_name': 'فایل فیلد', 'verbose_name_plural': 'فایل فیلد'},
        ),
        migrations.AlterField(
            model_name='filefield',
            name='file_direct',
            field=models.FileField(blank=True, null=True, upload_to=aparnik.utils.utils.document_directory_path, verbose_name='فایل'),
        ),
        migrations.AlterField(
            model_name='filefield',
            name='file_s3',
            field=s3direct.fields.S3DirectField(blank=True, null=True, verbose_name='فایل'),
        ),
        migrations.AlterField(
            model_name='filefield',
            name='password',
            field=models.CharField(blank=True, default='', max_length=64, null=True, verbose_name='گذرواژه'),
        ),
        migrations.AlterField(
            model_name='filefield',
            name='seconds',
            field=models.BigIntegerField(default=0, verbose_name='زمان'),
        ),
        migrations.AlterField(
            model_name='filefield',
            name='size',
            field=models.IntegerField(default=0, verbose_name='سایز فایل'),
        ),
        migrations.AlterField(
            model_name='filefield',
            name='title',
            field=models.CharField(blank=True, max_length=100, null=True, verbose_name='عنوان'),
        ),
        migrations.AlterField(
            model_name='filefield',
            name='type',
            field=models.CharField(blank=True, choices=[('M', 'فیلم'), ('V', 'صدا'), ('P', 'پی دی اف'), ('I', 'عکس')], max_length=1, null=True, verbose_name='نوع فایل'),
        ),
    ]



# -*- coding: utf-8 -*-


from django.db import models
from django.db.models import Q
from django.contrib.auth import get_user_model
from django.contrib.auth.models import Group
from django.core.exceptions import ValidationError
from django.utils.translation import ugettext_lazy as _
from django.utils.timezone import now
from django.db import transaction
from django.apps import apps

from aparnik.settings import Setting

User = get_user_model()


# Create your models here.
class ManagementActionsManager(models.Manager):
    def get_queryset(self):
        return super(ManagementActionsManager, self).get_queryset()


class ManagementActions(models.Model):
    title = models.CharField(max_length=50, verbose_name=_('Function title'))
    description = models.CharField(max_length=100, verbose_name=_('Description'))

    objects = ManagementActionsManager()

    def __str__(self):
        return str(self.title)

    class Meta:
        verbose_name = _('Admin action')
        verbose_name_plural = _('Admin actions')


class ManagementPermissionManager(models.Manager):
    def get_queryset(self):
        return super(ManagementPermissionManager, self).get_queryset()


class ManagementPermission(models.Model):
    # MANAGEMENT_PERMISSIONS = ['view','edit','readonly','delete']
    title = models.CharField(max_length=50, verbose_name=_('Title'))
    description = models.CharField(max_length=100, verbose_name=_('Description'))

    objects = ManagementPermissionManager()

    def __str__(self):
        return str(self.title)


class FieldListManager(models.Manager):
    def get_queryset(self):
        return super(FieldListManager, self).get_queryset()

    def update_fields(self, group_id):
        try:
            management_update_enable_status = Setting.objects.get(key='MANAGEMENT_DEFAULT_ENABLE_STATUS').get_value()
        except:
            management_update_enable_status = False

        managementActions = apps.get_model('managements', 'ManagementActions').objects.all()
        managementPermission = apps.get_model('managements', 'ManagementPermission').objects.all()
        # pk = 1;
        # First remove fields which not assigned to a group
        # FieldList.objects.filter(group__isnull=True).delete()

        for app in apps.get_models():
            for name in app._meta.get_fields(include_parents=True):
                with transaction.atomic():
                    field_type = name.get_internal_type()
                    if field_type in ['ForeignKey', 'ManyToManyField', 'OneToOneField', ]:
                        default_value = None

                    if field_type in ['FloatField', 'DecimalField', 'PositiveIntegerField', 'IntegerField',
                                      'BooleanField',
                                      'CharField',
                                      'TextField']:
                        default_value = None

                    if field_type in 'DateTimeField':
                        default_value = None

                    if field_type in 'AutoField':
                        default_value = None

                    if name.name in ['publish_day', 'publish_week', 'publish_month']:
                        default_value = None

                    field_exist = FieldList.objects.filter(model=app._meta.object_name, group_id=group_id,
                                                           name=name.name).count()

                    ## Currently django does not support bulk create with many-to-many felids.
                    if (field_exist == 0):
                        field = FieldList.objects.create(model=app._meta.object_name, name=name.name,
                                                         is_enable=management_update_enable_status, group_id=group_id,
                                                         is_sharable=False, default=default_value)

                        for permission in managementPermission:
                            field.permission.add(permission)
                        for action in managementActions:
                            field.actions.add(action)


class FieldList(models.Model):
    FIELD_LIST = []
    MODELS_LIST = []
    model = models.CharField(max_length=100, verbose_name=_('model'))
    name = models.CharField(max_length=100, verbose_name=_('Name'))
    group = models.ForeignKey(Group, blank=True, null=True, on_delete=models.CASCADE, verbose_name=_('group'))
    actions = models.ManyToManyField(ManagementActions, verbose_name=_('Actions'))
    permission = models.ManyToManyField(ManagementPermission, verbose_name=_('Management Permission'))
    is_enable = models.BooleanField(default=True, verbose_name=_('Is enable'))
    is_sharable = models.BooleanField(verbose_name=_('Is sharable'))
    default = models.CharField(max_length=200, null=True, blank=True, verbose_name=_('Default'))

    objects = FieldListManager()

    def __init__(self, *args, **kwargs):
        super(FieldList, self).__init__(*args, **kwargs)

    def __str__(self):
        return str(self.model + '.' + self.name)

    class Meta:
        verbose_name = _('Field')
        verbose_name_plural = _('Fields')


class ManagementManager(models.Manager):
    def get_queryset(self):
        return super(ManagementManager, self).get_queryset().all()

    def active(self):
        return self.get_queryset().filter(is_active=True, start_date__lte=now()).filter(
            (Q(end_date__gte=now()) | Q(end_date__isnull=True))).all()

    def get_permissions(self, group):
        return self.active().filter(group__in=[group], fields__is_enable=True).all()

    def get_actions(self, group):
        return self.active().filter(group=group).values_list('actions').all()

    def has_permit(self, group, allowed_permissions):
        if len(self.get_permissions(group)) < len(allowed_permissions):
            return False, 'Exceed parent permissions'
        for permit in allowed_permissions:
            if permit not in self.get_permissions(group):
                return False, permit
        return True

    def update_apps(self, group_id):
        for app in apps.get_app_configs():
            appModels = [model._meta.object_name for model in list(app.get_models(app.label))]
            fieldList = apps.get_model('managements', 'FieldList').objects.filter(model__in=appModels)
            if len(appModels) > 0 and len(fieldList) > 0:

                with transaction.atomic():
                    app_exist = Management.objects.filter(group_id=group_id, application=app.label).count()

                    if (app_exist == 0):
                        management = Management(group_id=group_id, application=app.label, start_date=now(),
                                                is_active=True)
                        management.save({'is_superuser': True})
                        for field in fieldList:
                                management.fields.add(field)
                    else:
                        management = Management.objects.filter(group_id=group_id, application=app.label).first()
                        for field in fieldList:
                            if field not in list(management.fields.all()):
                                management.fields.add(field)
                        management.save({'is_superuser': True})


class Management(models.Model):
    MODELS_NAME_LIST = []
    APPS_LABEL = []
    CONSTRAINTS_SET = []
    ROLES_SET = []

    group = models.ForeignKey(Group, on_delete=models.CASCADE, verbose_name=_('group'))
    application = models.CharField(max_length=100, verbose_name=_('Application'))
    fields = models.ManyToManyField(FieldList, verbose_name=_('fields'))

    parent_obj = models.ForeignKey('self', null=True, on_delete=models.CASCADE, verbose_name=_('Parent'))

    constraint = models.CharField(max_length=200, choices=CONSTRAINTS_SET, verbose_name=_('constraints'))

    start_date = models.DateTimeField(default=now, verbose_name=_('Start date'))

    end_date = models.DateTimeField(null=True, blank=True, verbose_name=_('End date'))

    is_active = models.BooleanField(default=True, verbose_name=_('Is Active'))

    objects = ManagementManager()

    def __str__(self):
        return str(self.group)

    def save(self, *args, **kwargs):
        if ('is_superuser' not in kwargs) and len(
                args) == 0:  # To handel requests from management migration or admin update
            (has_permit, unauthorized_permission) = Management.objects.has_permit(self.group, self.allowed_permissions)
        else:
            has_permit = True
            unauthorized_permission = []
        if has_permit or self.is_superuser or kwargs['is_superuser']:
            kwargs.pop('is_superuser', None)  # No need to pass this variable to upper class
            args = ()
            super(Management, self).save(*args, **kwargs)
        else:
            raise ValidationError(
                message=_('Not authorized to designate permission: %s' % str(unauthorized_permission)))

    class Meta:
        verbose_name = _('Admin Permission')
        verbose_name_plural = _('Admin Permissions')


class ManagementRolesManager(models.Manager):
    def get_queryset(self):
        return super(ManagementRolesManager, self).get_queryset()


class ManagementRoles(models.Model):
    title = models.CharField(max_length=30, verbose_name=_('Role name'))
    management_obj = models.ForeignKey(Management, on_delete=models.CASCADE, verbose_name=_('Management group'))

    objects = ManagementRolesManager()

    def __str__(self):
        return str(self.title)

    class Meta:
        verbose_name = _('Role')
        verbose_name_plural = _('Roles')



from rest_framework import serializers
from rest_polymorphic.serializers import PolymorphicSerializer
from aparnik.contrib.basemodels.api.serializers import BaseModelListSerializer, BaseModelDetailSerializer, ModelListPolymorphicSerializer
from aparnik.settings import aparnik_settings
from ..models import NotifyMe


UserSummeryListSerializer = aparnik_settings.USER_SUMMARY_LIST_SERIALIZER


class NotifyMeListSerializer(BaseModelListSerializer):
    user = serializers.SerializerMethodField()
    model = serializers.SerializerMethodField()

    def __init__(self, *args, **kwargs):
        super(NotifyMeListSerializer, self).__init__(*args, **kwargs)

        # if 'context' in kwargs:
        # self.fields['notification_type'] = NotificationTypeDetailSerializer(read_only=True, context=kwargs['context'])

    class Meta:
        model = NotifyMe
        fields = BaseModelListSerializer.Meta.fields + [
            'user',
            'model',
            'update_at',
        ]

    def get_user(self, obj):
        return UserSummeryListSerializer(obj.user_obj, many=False, read_only=True, context=self.context).data

    def get_model(self, obj):
        return ModelListPolymorphicSerializer(obj.model_obj, many=False, read_only=True, context=self.context).data



from django.conf.urls import url, include

from .views import OrderListAPIView, OrderDetailAPIView, AddProductOrderAPIView, AddOrderAddressAPIView, \
    ChargeWalletAPIView, RemoveProductOrderAPIView, AddProductOrderByWebsitesAPIView
from aparnik.packages.shops.payments.api.views import PayAPIView
from aparnik.packages.shops.coupons.api.views import CouponAPIView


app_name = 'orders'

urlpatterns = [
    url(r'^$', OrderListAPIView.as_view(), name='list'),
    url(r'^admin/', include('aparnik.packages.shops.orders.api.admin.urls', namespace='admin')),
    url(r'^wallet/charge/$', ChargeWalletAPIView.as_view(), name='charge-wallet'),
    url(r'^add/$', AddProductOrderAPIView.as_view(), name='add'),
    url(r'^websites/pay/$', AddProductOrderByWebsitesAPIView.as_view(), name='add-pay-by-websites'),
    url(r'^websites/pay/1.1/$', AddProductOrderByWebsitesAPIView.as_view(), name='add-pay-by-websites-new-version'),
    url(r'^(?P<id>\d+)/add-address/$', AddOrderAddressAPIView.as_view(), name='add-address'),
    url(r'^(?P<id>\d+)/add-item/$', AddProductOrderAPIView.as_view(), name='add-item'),
    url(r'^(?P<id>\d+)/remove-item/(?P<item_id>\d+)/$', RemoveProductOrderAPIView.as_view(), name='remove-item'),
    url(r'^(?P<id>\d+)/$', OrderDetailAPIView.as_view(), name='detail'),
    url(r'^(?P<id>\d+)/request-pay/$', PayAPIView.as_view(), name='request-pay'),
    url(r'^(?P<id>\d+)/coupon/$', CouponAPIView.as_view(), name='coupon'),
]


# Generated by Django 2.2 on 2019-07-17 10:17

from django.db import migrations

def add_keys(apps, schema_editor):
    '''
    We can't import the Post model directly as it may be a newer
    version than this migration expects. We use the historical version.
    '''
    Setting = apps.get_model('settings', 'Setting')
    key = ''
    try:
        key = 'PROJECT_DESCRIPTION'
        Setting.objects.get(key=key)
    except Exception:
        Setting.objects.create(
            title='توضیحات پروژه',
            key=key,
            value='',
            value_type='s',
            is_show=False,
            is_variable_in_home=False,
        )

    try:
        key = 'APPSTORE_LINK'
        Setting.objects.get(key=key)
    except Exception:
        Setting.objects.create(
            title='لینک اپ استور',
            key=key,
            value='',
            value_type='s',
            is_show=False,
            is_variable_in_home=False,
        )

    try:
        key = 'IOS_LINK'
        Setting.objects.get(key=key)
    except Exception:
        Setting.objects.create(
            title='لینک iOS',
            key=key,
            value='',
            value_type='s',
            is_show=False,
            is_variable_in_home=False,
        )

    try:
        key = 'GOOGLE_PLAY_LINK'
        Setting.objects.get(key=key)
    except Exception:
        Setting.objects.create(
            title='لینک گوگل پلی',
            key=key,
            value='',
            value_type='s',
            is_show=False,
            is_variable_in_home=False,
        )

    try:
        key = 'ANDROID_LINK'
        Setting.objects.get(key=key)
    except Exception:
        Setting.objects.create(
            title='لینک APK',
            key=key,
            value='',
            value_type='s',
            is_show=False,
            is_variable_in_home=False,
        )

    try:
        key = 'APPLICATION_IMAGE'
        Setting.objects.get(key=key)
    except Exception:
        Setting.objects.create(
            title='عکس نرم افزار',
            key=key,
            value='',
            value_type='s',
            is_show=False,
            is_variable_in_home=False,
        )



def remove_keys(apps, schema_editor):
    '''
    We can't import the Post model directly as it may be a newer
    version than this migration expects. We use the historical version.
    '''
    Setting = apps.get_model('settings', 'Setting')
    try:
        key = 'PROJECT_DESCRIPTION'
        Setting.objects.get(key=key).delete()
    except Exception:
        pass


    try:
        key = 'APPSTORE_LINK'
        Setting.objects.get(key=key).delete()
    except Exception:
        pass

    try:
        key = 'IOS_LINK'
        Setting.objects.get(key=key).delete()
    except Exception:
        pass

    try:
        key = 'GOOGLE_PLAY_LINK'
        Setting.objects.get(key=key).delete()
    except Exception:
        pass

    try:
        key = 'ANDROID_LINK'
        Setting.objects.get(key=key).delete()
    except Exception:
        pass

    try:
        key = 'APPLICATION_IMAGE'
        Setting.objects.get(key=key).delete()
    except Exception:
        pass

class Migration(migrations.Migration):

    dependencies = [
        ('settings', '0020_auto_20190623_0846'),
    ]

    operations = [
        migrations.RunPython(add_keys, reverse_code=remove_keys),
    ]



from django.conf.urls import url

from .views import DislikeReviewSetAPIView, DislikeListAPIView, LikeDetailAPIView

app_name = 'reviews'

urlpatterns = [
    url(r'^$', DislikeListAPIView.as_view(), name='list'),
    url(r'^set/$', DislikeReviewSetAPIView.as_view(), name='set'),
    url(r'^(?P<like_id>\d+)/$', LikeDetailAPIView.as_view(), name='detail'),
]


from django.db.models import Model
from rest_framework import serializers
from rest_polymorphic.serializers import PolymorphicSerializer

from aparnik.utils.utils import is_app_installed
from aparnik.api.serializers import ModelSerializer

from ..models import BaseModel, Tag


# Base Model List
class TagsDetailsSerializer(ModelSerializer):
    url = serializers.SerializerMethodField()
    resourcetype = serializers.SerializerMethodField()

    def __init__(self, *args, **kwargs):
        super(TagsDetailsSerializer, self).__init__(*args, **kwargs)

    class Meta:
        model = Tag
        fields = [
            'id',
            'url',
            'name',
            'resourcetype',
        ]

    def get_url(self, obj):
        return self.context['request'].build_absolute_uri(obj.get_api_uri())

    def get_resourcetype(self, obj):
        return 'Tag'


class BaseModelListSerializer(ModelSerializer):
    url = serializers.SerializerMethodField()
    url_share = serializers.SerializerMethodField()
    review_count = serializers.SerializerMethodField()
    qa_count = serializers.SerializerMethodField()
    url_review = serializers.SerializerMethodField()
    url_qa = serializers.SerializerMethodField()
    # is_bookmark = serializers.SerializerMethodField()
    url_bookmark_set = serializers.SerializerMethodField()
    url_review_add = serializers.SerializerMethodField()
    url_qa_add = serializers.SerializerMethodField()
    tags = serializers.SerializerMethodField()
    bookmark_count = serializers.SerializerMethodField()
    sort = serializers.SerializerMethodField()
    resourcetype = serializers.SerializerMethodField()

    # qa_count_string = serializers.SerializerMethodField()

    def __init__(self, *args, **kwargs):
        super(BaseModelListSerializer, self).__init__(*args, **kwargs)

    class Meta:
        model = BaseModel
        fields = [
            'id',
            'url',
            'url_share',
            'review_count',
            'review_count_string',
            'review_average',
            'qa_count',
            'qa_count_string',
            'url_review',
            'url_review_add',
            'url_qa',
            'url_qa_add',
            # 'is_bookmark',
            'url_bookmark_set',
            'tags',
            'sort',
            'visit_count',
            'visit_count_string',
            'bookmark_count',
            'bookmark_count_string',
            'resourcetype',
        ]

        read_only_fields = [
            'id',
            'url',
            'url_share',
            'is_bookmark',
            'review_average',
            'review_count',
            'review_count_string',
            'qa_count',
            'qa_count_string',
            'url_review',
            'url_qa',
            'tags',
            'sort',
            'visit_count',
            'visit_count_string',
            'bookmark_count',
            'bookmark_count_string',
            'resourcetype',
        ]

    def get_url(self, obj):
        return self.context['request'].build_absolute_uri(obj.get_api_uri())

    def get_url_share(self, obj):
        return self.context['request'].build_absolute_uri(obj.get_api_share_uri())

    def get_review_count(self, obj):
        return obj.review_count

    def get_qa_count(self, obj):
        return obj.qa_count

    def get_url_bookmark_set(self, obj):
        return self.context['request'].build_absolute_uri(obj.get_api_bookmark_set_uri())

    def get_url_review(self, obj):
        return self.context['request'].build_absolute_uri(obj.get_api_review_uri())

    def get_url_review_add(self, obj):
        return self.context['request'].build_absolute_uri(obj.get_api_review_add_uri())

    def get_url_qa(self, obj):
        return self.context['request'].build_absolute_uri(obj.get_api_qa_uri())

    def get_url_qa_add(self, obj):
        return self.context['request'].build_absolute_uri(obj.get_api_qa_add_uri())

    # def get_is_bookmark(self, obj):
    #     user = self.context['request'].user
    #     return obj.is_bookmark(user=user)
    #

    def get_tags(self, obj):
        # if not obj.tags:
        #     return None
        # return TagsDetailsSerializer(obj.tags, many=True, read_only=True, context=self.context).data
        # worse performance
        return None

    def get_sort(self, obj):
        if 'segment_obj' in self.context:
            objs = self.context['segment_obj'].segment_sort_segment.filter(model_obj=obj)
            if objs.exists():
                return objs.first().sort
        return obj.sort

    def get_bookmark_count(self, obj):
        return obj.bookmark_count

    def get_resourcetype(self, obj):
        return obj._meta.object_name


# Base Model Details
class BaseModelDetailSerializer(BaseModelListSerializer):

    # share_string = serializers.SerializerMethodField()
    # review_summaries = serializers.SerializerMethodField()
    # is_notify_me = serializers.SerializerMethodField()
    # url_notify_me_set = serializers.SerializerMethodField()

    def __init__(self, *args, **kwargs):
        super(BaseModelListSerializer, self).__init__(*args, **kwargs)

    class Meta:
        model = BaseModel
        fields = BaseModelListSerializer.Meta.fields + [
            # 'share_string',
            # 'review_summaries',
            # 'is_notify_me',
            # 'url_notify_me_set',
        ]
        read_only_fields = BaseModelListSerializer.Meta.read_only_fields + [
        ]

    # def get_share_string(self, obj):
    #     # TODO: fix it
    #     return 'This content is ready for sharing'
    #
    #
    # def get_review_summaries(self, obj):
    #     from aparnik.contrib.reviews.api.serializers import ReviewSummaryListSerializer
    #     return ReviewSummaryListSerializer(obj.review_summaries, many=True, read_only=True, context=self.context).data
    #
    # def get_is_notify_me(self, obj):
    #     user = self.context['request'].user
    #     return obj.is_notify(user=user)
    #
    # def get_url_notify_me_set(self, obj):
    #     return self.context['request'].build_absolute_uri(obj.get_api_notify_me_set_uri())


# Base Model List Polymorphic
class ModelListPolymorphicSerializer(PolymorphicSerializer):
    def __init__(self, *args, **kwargs):

        model_serializer_mapping = {
            BaseModel: BaseModelListSerializer
        }

        # File Fileld
        if is_app_installed('aparnik.contrib.filefields'):
            from aparnik.contrib.filefields.models import FileField
            from aparnik.contrib.filefields.api.serializers import FileFieldListSerailizer
            model_serializer_mapping[FileField] = FileFieldListSerailizer

        # Page
        if is_app_installed('aparnik.contrib.pages'):
            from aparnik.contrib.pages.models import Page
            from aparnik.contrib.pages.api.serializers import PageListPolymorphicSerializer
            model_serializer_mapping[Page] = PageListPolymorphicSerializer

        # Slider
        if is_app_installed('aparnik.contrib.sliders'):
            from aparnik.contrib.sliders.models import Slider
            from aparnik.contrib.sliders.api.serializers import SliderDetailsPolymorphicSerializer
            model_serializer_mapping[Slider] = SliderDetailsPolymorphicSerializer

        # Review
        if is_app_installed('aparnik.contrib.reviews'):
            from aparnik.contrib.reviews.models import BaseReview
            from aparnik.contrib.reviews.api.serializers import BaseReviewListPolymorphicSerializer
            model_serializer_mapping[BaseReview] = BaseReviewListPolymorphicSerializer

        # user address
        if is_app_installed('aparnik.contrib.addresses'):
            from aparnik.contrib.addresses.models import UserAddress
            from aparnik.contrib.addresses.api.serializers import AddressListSerializer
            model_serializer_mapping[UserAddress] = AddressListSerializer

        # bank accounts
        if is_app_installed('aparnik.contrib.bankaccounts'):
            from aparnik.contrib.bankaccounts.models import BankName, BankAccount
            from aparnik.contrib.bankaccounts.api.serializers import BankNameListSerializer, BankAccountListSerializer
            model_serializer_mapping[BankName] = BankNameListSerializer
            model_serializer_mapping[BankAccount] = BankAccountListSerializer

        # Button
        if is_app_installed('aparnik.contrib.buttons'):
            from aparnik.contrib.buttons.models import Button
            from aparnik.contrib.buttons.api.serializers import ButtonListSerializer
            model_serializer_mapping[Button] = ButtonListSerializer

        # Segments
        if is_app_installed('aparnik.contrib.segments'):
            from aparnik.contrib.segments.models import BaseSegment
            from aparnik.contrib.segments.api.serializers import BaseSegmentListPolymorphicSerializer
            model_serializer_mapping[BaseSegment] = BaseSegmentListPolymorphicSerializer

        # Category
        if is_app_installed('aparnik.contrib.categories'):
            from aparnik.contrib.categories.models import Category
            from aparnik.contrib.categories.api.serializers import CategoryListPolymorphicSerializer
            model_serializer_mapping[Category] = CategoryListPolymorphicSerializer

        # Notification
        if is_app_installed('aparnik.contrib.notifications'):
            from aparnik.contrib.notifications.models import Notification
            from aparnik.contrib.notifications.api.serializers import NotificationListSerializer
            model_serializer_mapping[Notification] = NotificationListSerializer

        # Social Network
        if is_app_installed('aparnik.contrib.socials'):
            from aparnik.contrib.socials.models import SocialNetwork
            from aparnik.contrib.socials.api.serializers import SocialNetworkDestailSerializer
            model_serializer_mapping[SocialNetwork] = SocialNetworkDestailSerializer

        # Prdocut
        if is_app_installed('aparnik.packages.shops.products'):
            from aparnik.packages.shops.products.models import Product
            from aparnik.packages.shops.products.api.serializers import ProductListPolymorphicSerializer
            model_serializer_mapping[Product] = ProductListPolymorphicSerializer

        # Order
        if is_app_installed('aparnik.packages.shops.orders'):
            from aparnik.packages.shops.orders.models import Order, OrderItem
            from aparnik.packages.shops.orders.api.serializers import OrderSummarySerializer, OrderItemDetailSerializer
            model_serializer_mapping[Order] = OrderSummarySerializer
            model_serializer_mapping[OrderItem] = OrderItemDetailSerializer

        # Payment
        if is_app_installed('aparnik.packages.shops.payments'):
            from aparnik.packages.shops.payments.models import Payment
            from aparnik.packages.shops.payments.api.serializers import PaymentSummarySerializer
            model_serializer_mapping[Payment] = PaymentSummarySerializer

        # Co Sale
        if is_app_installed('aparnik.packages.shops.cosales'):
            from aparnik.packages.shops.cosales.models import CoSale
            from aparnik.packages.shops.cosales.api.serializers import CoSaleListSerializer
            model_serializer_mapping[CoSale] = CoSaleListSerializer

        # Teacher
        if is_app_installed('aparnik.packages.educations.teachers'):
            from aparnik.packages.educations.teachers.models import Teacher
            from aparnik.packages.educations.teachers.api.serializers import TeacherListPolymorphicSerializer
            model_serializer_mapping[Teacher] = TeacherListPolymorphicSerializer

        # Base Educations
        if is_app_installed('aparnik.packages.educations.educations'):
            from aparnik.packages.educations.educations.models import BaseEducation
            from aparnik.packages.educations.educations.api.serializers import BaseEducationListPolymorphicSerializer
            model_serializer_mapping[BaseEducation] = BaseEducationListPolymorphicSerializer

        # news
        if is_app_installed('aparnik.packages.news'):
            from aparnik.packages.news.models import News
            from aparnik.packages.news.api.serializers import NewsListSerializer
            model_serializer_mapping[News] = NewsListSerializer

        self.model_serializer_mapping = model_serializer_mapping
        super(ModelListPolymorphicSerializer, self).__init__(*args, **kwargs)

    model_serializer_mapping = {}


# Base Model Details Polymorphic
class ModelDetailsPolymorphicSerializer(PolymorphicSerializer):
    def __init__(self, *args, **kwargs):

        model_serializer_mapping = {
            BaseModel: BaseModelDetailSerializer
        }

        # File Fileld
        if is_app_installed('aparnik.contrib.filefields'):
            from aparnik.contrib.filefields.models import FileField
            from aparnik.contrib.filefields.api.serializers import FileFieldDetailsSerailizer
            model_serializer_mapping[FileField] = FileFieldDetailsSerailizer

        # Page
        if is_app_installed('aparnik.contrib.pages'):
            from aparnik.contrib.pages.models import Page
            from aparnik.contrib.pages.api.serializers import PageDetailsPolymorphicSerializer
            model_serializer_mapping[Page] = PageDetailsPolymorphicSerializer

        # Slider
        if is_app_installed('aparnik.contrib.sliders'):
            from aparnik.contrib.sliders.models import Slider
            from aparnik.contrib.sliders.api.serializers import SliderDetailsPolymorphicSerializer
            model_serializer_mapping[Slider] = SliderDetailsPolymorphicSerializer

        # Review
        if is_app_installed('aparnik.contrib.reviews'):
            from aparnik.contrib.reviews.models import BaseReview
            from aparnik.contrib.reviews.api.serializers import BaseReviewDetailsPolymorphicSerializer
            model_serializer_mapping[BaseReview] = BaseReviewDetailsPolymorphicSerializer

        # user address
        if is_app_installed('aparnik.contrib.addresses'):
            from aparnik.contrib.addresses.models import UserAddress
            from aparnik.contrib.addresses.api.serializers import AddressDetailSerializer
            model_serializer_mapping[UserAddress] = AddressDetailSerializer

        # bank accounts
        if is_app_installed('aparnik.contrib.bankaccounts'):
            from aparnik.contrib.bankaccounts.models import BankName, BankAccount
            from aparnik.contrib.bankaccounts.api.serializers import BankNameDetailSerializer, \
                BankAccountDetailSerializer
            model_serializer_mapping[BankName] = BankNameDetailSerializer
            model_serializer_mapping[BankAccount] = BankAccountDetailSerializer

        # Button
        if is_app_installed('aparnik.contrib.buttons'):
            from aparnik.contrib.buttons.models import Button
            from aparnik.contrib.buttons.api.serializers import ButtonDetailsSerializer
            model_serializer_mapping[Button] = ButtonDetailsSerializer

        # Segments
        if is_app_installed('aparnik.contrib.segments'):
            from aparnik.contrib.segments.models import BaseSegment
            from aparnik.contrib.segments.api.serializers import BaseSegmentDetailsPolymorphicSerializer
            model_serializer_mapping[BaseSegment] = BaseSegmentDetailsPolymorphicSerializer

        # Category
        if is_app_installed('aparnik.contrib.categories'):
            from aparnik.contrib.categories.models import Category
            from aparnik.contrib.categories.api.serializers import CategoryDetailsPolymorphicSerializer
            model_serializer_mapping[Category] = CategoryDetailsPolymorphicSerializer

        # Notification
        if is_app_installed('aparnik.contrib.notifications'):
            from aparnik.contrib.notifications.models import Notification
            from aparnik.contrib.notifications.api.serializers import NotificationDetailSerializer
            model_serializer_mapping[Notification] = NotificationDetailSerializer

        # Social Network
        if is_app_installed('aparnik.contrib.socials'):
            from aparnik.contrib.socials.models import SocialNetwork
            from aparnik.contrib.socials.api.serializers import SocialNetworkDestailSerializer
            model_serializer_mapping[SocialNetwork] = SocialNetworkDestailSerializer

        # Prdocut
        if is_app_installed('aparnik.packages.shops.products'):
            from aparnik.packages.shops.products.models import Product
            from aparnik.packages.shops.products.api.serializers import ProductDetailsPolymorphicSerializer
            model_serializer_mapping[Product] = ProductDetailsPolymorphicSerializer

        # Order
        if is_app_installed('aparnik.packages.shops.orders'):
            from aparnik.packages.shops.orders.models import Order, OrderItem
            from aparnik.packages.shops.orders.api.serializers import OrderDetailSerializer, OrderItemDetailSerializer
            model_serializer_mapping[Order] = OrderDetailSerializer
            model_serializer_mapping[OrderItem] = OrderItemDetailSerializer

        # Payment
        if is_app_installed('aparnik.packages.shops.payments'):
            from aparnik.packages.shops.payments.models import Payment
            from aparnik.packages.shops.payments.api.serializers import PaymentDetailSerializer
            model_serializer_mapping[Payment] = PaymentDetailSerializer

        # Co Sale
        if is_app_installed('aparnik.packages.shops.cosales'):
            from aparnik.packages.shops.cosales.models import CoSale
            from aparnik.packages.shops.cosales.api.serializers import CoSaleDetailsSerializer
            model_serializer_mapping[CoSale] = CoSaleDetailsSerializer

        # Teacher
        if is_app_installed('aparnik.packages.educations.teachers'):
            from aparnik.packages.educations.teachers.models import Teacher
            from aparnik.packages.educations.teachers.api.serializers import TeacherDetailsPolymorphicSerializer
            model_serializer_mapping[Teacher] = TeacherDetailsPolymorphicSerializer

        # Base Educations
        if is_app_installed('aparnik.packages.educations.educations'):
            from aparnik.packages.educations.educations.models import BaseEducation
            from aparnik.packages.educations.educations.api.serializers import BaseEducationDetailsPolymorphicSerializer
            model_serializer_mapping[BaseEducation] = BaseEducationDetailsPolymorphicSerializer

        # news
        if is_app_installed('aparnik.packages.news'):
            from aparnik.packages.news.models import News
            from aparnik.packages.news.api.serializers import NewsDetailSerializer
            model_serializer_mapping[News] = NewsDetailSerializer

        self.model_serializer_mapping = model_serializer_mapping
        super(ModelDetailsPolymorphicSerializer, self).__init__(*args, **kwargs)

    model_serializer_mapping = {}



from django.conf.urls import url, include

from .views import SignS3, FileFieldCreateAPIView, FileFieldListAPIView

app_name = 'filefields'

urlpatterns = [
    url(r'^filefileds-list/', FileFieldListAPIView.as_view(), name='list'),
    url(r'^sign-s3/', SignS3.as_view(), name='s3-sign'),
    url(r'^create/', FileFieldCreateAPIView.as_view(), name='create'),
]


from azbankgateways.models import Bank
from django.db import models
from django.utils.translation import gettext_lazy as _

from .payments import Payment


class PaymentBank(models.Model):
    payment = models.ForeignKey(
        Payment,
        on_delete=models.PROTECT,
        verbose_name=_('Payment')
    )
    bank_record = models.ForeignKey(
        Bank,
        on_delete=models.PROTECT,
        verbose_name=_('Bank record')
    )

    created_at = models.DateTimeField(auto_now_add=True, verbose_name=_('Created at'))
    update_at = models.DateTimeField(auto_now=True, verbose_name=_('Update at'))

    class Meta:
        unique_together = [['bank_record']]
        verbose_name = _('Payment Bank')
        verbose_name_plural = _('Payments Bank')



# -*- coding: utf-8 -*-


from django.core.management import BaseCommand
from django.db.models import Count, Subquery, OuterRef, F
from django.db.models.functions import Coalesce
from django.utils.timezone import now
from dateutil.relativedelta import relativedelta

from aparnik.settings import aparnik_settings, Setting
from aparnik.contrib.counters.models import Counter
from aparnik.contrib.basemodels.models import BaseModel

UserSummeryListSerializer = aparnik_settings.USER_SUMMARY_LIST_SERIALIZER


class Command(BaseCommand):
    # Show this when the user types help
    help = "Updates visit counter for models based on a variable in settings app"

    # A command must define handle()
    def handle(self, *args, **options):
        start_time = now()

        try:
            is_count_per_view = Setting.objects.get(key='VISIT_COUNT_PER_VIEW').get_value()
        except:
            is_count_per_view = False

        # Update visit for each user
        if (is_count_per_view):
            BaseModel.objects.update(
                visit_count=Coalesce(Subquery(
                    BaseModel.objects.active().filter(
                        counter__model_obj=OuterRef('id'), counter__action='v'
                    ).annotate(count=(Count('counter__create_date') or 0)).values('count')

                ), 0)
            )

        if (not is_count_per_view):
            # This case devided into two query to improve performance
            # count normal users
            BaseModel.objects.update(
                visit_count=Coalesce(Subquery(
                    BaseModel.objects.active().filter(
                        counter__model_obj=OuterRef('id'), counter__action='v'
                    ).annotate(count=

                               Count('counter__user_obj', distinct=True)

                               ).values('count')

                ), 0)
            )

            # Add anonymous users visit count
            BaseModel.objects.update(
                visit_count=F('visit_count') + Coalesce(Subquery(
                    BaseModel.objects.active().filter(
                        counter__model_obj=OuterRef('id'), counter__user_obj_id__isnull=True, counter__action='v'
                    ).annotate(count=

                               Count('counter__user_obj', distinct=True)

                               ).values('count')

                ), 0)
            )

        finished_time = now()
        # print(' Success. \n Counter values for models updated; setting flag VISIT_COUNT_PER_VIEW = ' + str(
        #     is_count_per_view))

        print(('counters updated %s - time long: %ss.' % (now(), relativedelta(finished_time, start_time).seconds)))



# Generated by Django 2.2 on 2020-01-07 14:03

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('aboutus', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='information',
            name='short_blogs',
            field=models.ManyToManyField(blank=True, to='shortblogs.ShortBlog', verbose_name='Content Segment'),
        ),
    ]



# -*- coding: utf-8 -*-


from django.db import models
from django.utils.translation import ugettext_lazy as _

from aparnik.contrib.basemodels.models import BaseModel, BaseModelManager
from aparnik.contrib.filefields.models import FileField
from aparnik.contrib.socials.models import SocialNetwork
from aparnik.contrib.pages.models import Page
from aparnik.contrib.segments.models import BaseSegment, BaseSegmentManager


# Create your models here.
class SliderManager(BaseModelManager):

    def get_queryset(self):
        return super(SliderManager, self).get_queryset()

    def active(self):
        return self.get_queryset().filter(is_active=True)

    def home(self):
        return self.active().filter(is_home=True)

    def library(self):
        return self.active().filter(is_library=True)

    def page(self, page):
        return self.active().filter(pages=page)


class Slider(BaseModel):
    name = models.CharField(max_length=50, null=True, blank=True, verbose_name=_('Name'))
    image = models.ForeignKey(FileField, on_delete=models.CASCADE, verbose_name=_('Image'))
    is_active = models.BooleanField(default=True, verbose_name=_('Is Active'))

    objects = SliderManager()

    def __str__(self):
        return self.name

    class Meta:
        verbose_name = _('Slide Show')
        verbose_name_plural = _('Slide Shows')


class SliderImage(Slider):

    objects = SliderManager()

    def __str__(self):
        return self.name

    class Meta:
        verbose_name = _('Image Slide Show')
        verbose_name_plural = _('Image Slide Shows')


class SliderVideo(Slider):

    video = models.ForeignKey(FileField, on_delete=models.CASCADE, verbose_name=_('Video'))

    objects = SliderManager()

    def __str__(self):
        return self.name

    class Meta:
        verbose_name = _('Video Slider')
        verbose_name_plural = _('Video Sliders')


class SliderSocialNetwork(Slider):

    social = models.ForeignKey(SocialNetwork, on_delete=models.CASCADE, verbose_name=_('Social'))

    objects = SliderManager()

    def __str__(self):
        return self.name

    class Meta:
        verbose_name = _('Social Network Slider')
        verbose_name_plural = _('Social Network Sliders')


class SliderLink(Slider):

    link = models.URLField(verbose_name=_('Link'))

    objects = SliderManager()

    def __str__(self):
        return self.name

    class Meta:
        verbose_name = _('Link Slider')
        verbose_name_plural = _('Link Sliders')


class SliderBaseModel(Slider):

    model = models.ForeignKey(BaseModel, on_delete=models.CASCADE, verbose_name=_('Model'))

    objects = SliderManager()

    def __str__(self):
        return self.name

    class Meta:
        verbose_name = _('Model Slider')
        verbose_name_plural = _('Model Sliders')


# Slider Segment Model
class SliderSegment(BaseSegment):

    objects = BaseSegmentManager()

    def __str__(self):
        return super(SliderSegment, self).__str__()

    class Meta:
        verbose_name = _('Slider Segment')
        verbose_name_plural = _('Sliders Segments')



from rest_framework.generics import RetrieveAPIView, CreateAPIView, UpdateAPIView, ListAPIView, \
    RetrieveUpdateDestroyAPIView
from rest_framework.permissions import ( AllowAny, IsAuthenticated )

from aparnik.contrib.users.api.permissions import IsAdminPermission
from .serializers import ContactUsDestailSerializer
from ..models import ContactUs


class ContactUsListAPIView(ListAPIView):
    serializer_class = ContactUsDestailSerializer
    permission_classes = [IsAdminPermission]
    queryset = ContactUs.objects.active().order_by('-update_at')


class ContactUsDetailAPIView(RetrieveUpdateDestroyAPIView):
    serializer_class = ContactUsDestailSerializer
    permission_classes = [IsAdminPermission]
    lookup_field = 'id'
    lookup_url_kwarg = 'id'

    def get_queryset(self):
        return ContactUs.objects.active()


class ContactUsCreateAPIView(CreateAPIView):
    serializer_class = ContactUsDestailSerializer
    permission_classes = [AllowAny]
    queryset = ContactUs.objects.active()



# Generated by Django 2.2 on 2020-04-05 16:28

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('aboutus', '0003_auto_20200405_1627'),
    ]

    operations = [
        migrations.AlterField(
            model_name='information',
            name='socials',
            field=models.ManyToManyField(blank=True, to='socials.SocialNetwork', verbose_name='Socials'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-12-14 13:30


from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('basemodels', '0008_auto_20181211_1552'),
    ]

    operations = [
        migrations.AddField(
            model_name='basemodel',
            name='sort',
            field=models.IntegerField(default=0, verbose_name='\\u0645\\u0631\\u062a\\u0628 \\u0633\\u0627\\u0632\\u06cc'),
        ),
    ]



from django.conf.urls import url

from .views import payment, callback_view

app_name = 'payments'

urlpatterns = [
    url(r'^(?P<uuid>[0-9a-f-]+)/pay/$', payment, name='payment'),
    url(r'^verify/$', callback_view, name='call-back'),
    # url(r'^(?P<id>\d+)/pay/$', TransactionPayAPIView.as_view(), name='pay'),
    # url(r'^(?P<id>\d+)/$', TransactionDetailAPIView.as_view(), name='detail'),

]



# -*- coding: utf-8 -*-


from django.core.management import BaseCommand
from django.core.serializers.json import DjangoJSONEncoder
from django.contrib.auth import get_user_model
from django.utils.timezone import now
from dateutil.relativedelta import relativedelta
from django.utils.text import Truncator


import json

from fcm_django.models import FCMDevice
from aparnik.utils.utils import get_request
from aparnik.contrib.settings.models import Setting
from aparnik.contrib.notifications.models import Notification
from aparnik.contrib.basemodels.api.serializers import ModelListPolymorphicSerializer
from aparnik.settings import aparnik_settings

User = get_user_model()
UserSummeryListSerializer = aparnik_settings.USER_SUMMARY_LIST_SERIALIZER


class Command(BaseCommand):
    # Show this when the user types help
    help = "send notification"

    # A command must define handle()
    def handle(self, *args, **options):

        start_time = now()

        finished_time = now()

        print(('send notifications %s - time long: %ss.' % (now(), relativedelta(finished_time, start_time).seconds)))



"""Custom model fields."""

import json
from django.db import models


class JSONField(models.TextField):
    """Add's JSON capablilties (retrieval) to Django's TextField."""

    def get_prep_value(self, value):
        """Convert the dictionary to JSON, which is saved as text."""
        db_value = super(JSONField, self).get_prep_value(value)

        return json.dumps(db_value)

    def from_db_value(self, value, expression, connection, *args, **kwargs):
        """Convert the JSON back to a dictionary."""
        try:
            return json.loads(value)
        except json.JSONDecodeError:
            return value

    def to_python(self, value):
        """Simply returns the dictionary"""
        return value


class ListField(models.CharField):
    """Emulate django.contrib.postgres ArrayField."""

    def get_prep_value(self, value):
        """Convert the list/tuple to a comma separated string."""
        super(ListField, self).get_prep_value(value)

        return ','.join(value)

    def from_db_value(self, value, expression, connection, *args, **kwargs):
        """Return the value from the database as a python list."""

        return value.split(',')



# -*- coding: utf-8 -*-


from django.db import models
from django.core.validators import ValidationError
from django.urls import reverse
from django.utils.translation import ugettext_lazy as _

from aparnik.utils.utils import is_app_installed
from aparnik.contrib.basemodels.models import BaseModel, BaseModelManager, BaseModelQueryset
from aparnik.contrib.segments.models import BaseSegment, BaseSegmentManager
from aparnik.contrib.filefields.models import FileField
from aparnik.contrib.pages.models import Page


# Category Queryset
class CategoryQueryset(BaseModelQueryset):
    def show(self):
        return self.filter(is_show=True)


# Category Manager
class CategoryManager(BaseModelManager):
    def get_queryset(self):
        return CategoryQueryset(self.model, using=self._db)

    def show(self):
        return self.active().show()

    def page(self, page):
        return self.active().filter(pages=page)

    def first_level(self):
        return self.active().filter(parent_obj__isnull=True)

    def level(self, level):
        return self.active().filter(parent_obj__id=level)

    def home(self):
        from aparnik.contrib.settings.models import Setting
        try:
            setting = Setting.objects.get(key='CATEGORIES_HOME_ROOT')
            return self.active().filter(parent_obj__id=setting.get_value())
        except Exception as e:
            pass
        return self.none()

    def books(self):
        if is_app_installed('aparnik.packages.educations.books'):
            from aparnik.contrib.settings.models import Setting
            try:
                setting = Setting.objects.get(key='CATEGORIES_BOOKS_ROOT')
                return self.active().filter(parent_obj__id=setting.get_value())
            except Exception as e:
                pass
        return self.none()

    def courses(self):
        if is_app_installed('aparnik.packages.educations.courses'):
            from aparnik.contrib.settings.models import Setting
            try:
                setting = Setting.objects.get(key='CATEGORIES_COURSES_ROOT')
                return self.active().filter(parent_obj__id=setting.get_value())
            except Exception as e:
                pass
        return self.none()

    def childs(self, category_obj):
        return self.active().filter(parent_obj=category_obj)

    def childs_count(self, category_obj):
        return self.childs(category_obj).count()


# Category Model
class Category(BaseModel):
    title = models.CharField(max_length=255, verbose_name=_('Title'))
    description = models.TextField(blank=True, verbose_name=_('Description'))
    image = models.ForeignKey(FileField, on_delete=models.CASCADE, verbose_name=_('Image'))
    parent_obj = models.ForeignKey('self', blank=True, null=True, on_delete=models.CASCADE, verbose_name=_('Parent'))
    # pages = models.ManyToManyField(Page, blank=True, verbose_name=_('Page'))
    is_show = models.BooleanField(default=True, verbose_name=_('Is Show'))

    objects = CategoryManager()

    class Meta:
        verbose_name = _('Category')
        verbose_name_plural = _('Categories')

    def __str__(self):
        return self.title

    # def get_api_uri(self):
    #     return reverse("api:category:detail", args=[self.id])

    def get_api_model_list_uri(self):
        return reverse('aparnik-api:categories:list-model', args=[self.id])

    def get_api_child_uri(self):
        return reverse('aparnik-api:categories:list-childs', args=[self.id])


# Course Segment Model
class CategorySegment(BaseSegment):

    objects = BaseSegmentManager()

    def __str__(self):
        return super(CategorySegment, self).__str__()

    class Meta:
        verbose_name = _('Category Segment')
        verbose_name_plural = _('Category Segments')



from django.conf.urls import url, include

from .views import ProductSortAdminAPIView, ProductAdminListAPIView

app_name = 'products'

urlpatterns = [
    url(r'^$', ProductAdminListAPIView.as_view(), name='list'),
    url(r'^sort/$', ProductSortAdminAPIView.as_view(), name='sort'),
]



# -*- coding: utf-8 -*-


from django.contrib import admin

from aparnik.contrib.users.admin import get_update_at
from aparnik.utils.utils import is_app_installed
from .models import SocialNetwork
from aparnik.contrib.basemodels.admin import BaseModelAdmin
# Register your models here.


class SocialNetworkAdmin(BaseModelAdmin):
    fields = ['title', 'link', 'icon', 'value', 'android_app_shortcut', 'ios_app_shortcut']
    list_display = ['title', 'link', 'icon', get_update_at]
    list_filter = ['created_at']
    search_fields = ['title']
    readonly_fields = []
    raw_id_fields = ['icon']
    exclude = []
    dynamic_raw_id_fields = []

    def __init__(self, *args, **kwargs):
        Klass = SocialNetworkAdmin
        Klass_parent = BaseModelAdmin

        super(Klass, self).__init__(*args, **kwargs)
        self.fields = Klass_parent.fields + self.fields
        self.list_display = Klass_parent.list_display + self.list_display
        self.list_filter = Klass_parent.list_filter + self.list_filter
        self.search_fields = Klass_parent.search_fields + self.search_fields
        self.exclude = Klass_parent.exclude + self.exclude
        self.dynamic_raw_id_fields = Klass_parent.dynamic_raw_id_fields + self.dynamic_raw_id_fields
        self.raw_id_fields = Klass_parent.raw_id_fields + self.raw_id_fields

    class Meta:
        model = SocialNetwork


if is_app_installed('aparnik.contrib.segments'):
    from aparnik.contrib.segments.admin import SegmentAdmin
    from .models import SocialNetworkSegment

    class SocialNetworkSegmentAdmin(SegmentAdmin):
        fields = []
        list_display = []
        search_fields = []
        list_filter = []
        raw_id_fields = []
        exclude = []
        dynamic_raw_id_fields = []

        def __init__(self, *args, **kwargs):
            Klass = SocialNetworkSegmentAdmin
            Klass_parent = SegmentAdmin

            super(Klass, self).__init__(*args, **kwargs)
            self.fields = Klass_parent.fields + self.fields
            self.list_display = Klass_parent.list_display + self.list_display
            self.list_filter = Klass_parent.list_filter + self.list_filter
            self.search_fields = Klass_parent.search_fields + self.search_fields
            self.exclude = Klass_parent.exclude + self.exclude
            self.dynamic_raw_id_fields = Klass_parent.dynamic_raw_id_fields + self.dynamic_raw_id_fields
            self.raw_id_fields = Klass_parent.raw_id_fields + self.raw_id_fields

        class Meta:
            model = SocialNetworkSegment


    admin.site.register(SocialNetworkSegment, SocialNetworkSegmentAdmin)


admin.site.register(SocialNetwork, SocialNetworkAdmin)



"""Models for the chat app."""
from datetime import date
from uuid import uuid4

from django.core.exceptions import ValidationError
from django.db.models import Q, Max, DateTimeField
from django.db.models.functions import Coalesce, Cast
from django.db.models.signals import post_save
from django.urls import reverse
from django.utils.translation import ugettext_lazy as _
from aparnik.contrib.filefields.models import FileField
from django.db import models
from django.contrib.auth import get_user_model
from django_enumfield import enum

User = get_user_model()


class ChatSessionTypeEnum(enum.Enum):
    PRIVATE = 0
    GROUP = 1
    CHANNEL = 2

    labels = {
        PRIVATE: _('PRIVATE'),
        GROUP: _('GROUP'),
        CHANNEL: _('CHANNEL')
    }


class TrackableDateModel(models.Model):
    """Abstract model to Track the creation/updated date for a model."""

    created_at = models.DateTimeField(auto_now_add=True)
    update_at = models.DateTimeField(auto_now=True)

    class Meta:
        abstract = True

    @property
    def resourcetype(self):
        return self._meta.object_name


def _generate_unique_uri():
    """Generates a unique uri for the chat session."""
    return str(uuid4()).replace('-', '')[:15]


class ChatSessionManager(models.Manager):
    def get_queryset(self):
        return super(ChatSessionManager, self).get_queryset().select_related()

    def user_session(self, user):
        queryset_first = self.get_queryset().filter(
            pk__in=ChatSessionMember.objects.filter(user=user).values_list('chat_session', flat=True).distinct())
        queryset = (queryset_first | self.get_queryset().filter(owner=user)).distinct().annotate(
            message_created_at=Coalesce(Max('messages__created_at'),
                                        Cast(date(2002, 12, 31), DateTimeField()))
        )
        return queryset

    def private(self, owner, user):
        """
        You can add private chat one by one.
        :param owner: User first time create the chat
        :param user: User add to this chat
        :return: ChatSession instance
        """
        username_first = owner.username
        username_second = user.username
        queryset = self.get_queryset().filter(Q(type=ChatSessionTypeEnum.PRIVATE), Q(owner__username=username_first,
                                                                                     members__user__username=username_second))
        queryset = (queryset | self.get_queryset().filter(Q(type=ChatSessionTypeEnum.PRIVATE),
                                                          Q(owner__username=username_second,
                                                            members__user__username=username_first))).distinct()
        if queryset.exists():
            return queryset.first()
        chat_session = ChatSession.objects.create(
            owner=owner,
            type=ChatSessionTypeEnum.PRIVATE,
        )
        ChatSessionMember.objects.create(
            chat_session=chat_session,
            user=user,
        )
        return chat_session


class ChatSession(TrackableDateModel):
    """
    A Chat Session.

    The uri's are generated by taking the first 15 characters from a UUID
    """

    owner = models.ForeignKey(User, on_delete=models.PROTECT, verbose_name=_('Owner'))
    uri = models.URLField(default=_generate_unique_uri, verbose_name=_('Uri'))
    type = enum.EnumField(ChatSessionTypeEnum, default=ChatSessionTypeEnum.GROUP,
                          verbose_name=_('Type'))
    cover_obj = models.ForeignKey(FileField, null=True, blank=True, on_delete=models.PROTECT, verbose_name=_('Cover'))
    title = models.CharField(max_length=255, default='', blank=True, verbose_name=_('Title'))

    objects = ChatSessionManager()

    def __str__(self):
        return self.uri

    def full_clean(self, exclude=None, validate_unique=True):
        if self.type != ChatSessionTypeEnum.PRIVATE and (self.title == '' or self.title is None):
            raise ValidationError(
                {'title': [_('This filed is required.'), ], }, )

    def save(self, *args, **kwargs):
        self.full_clean()
        return super(ChatSession, self).save(*args, **kwargs)

    def get_api_uri(self):
        return reverse('aparnik-api:chats:details', args=[self.uri])

    def get_api_mark_as_read_uri(self):
        return reverse('aparnik-api:chats:mark-as-read', args=[self.uri])

    def get_member_api_uri(self):
        return reverse('aparnik-api:chats:members', args=[self.uri])

    def get_messages_api_uri(self):
        return reverse('aparnik-api:chats:messages', args=[self.uri])

    def get_messages_create_api_uri(self):
        return reverse('aparnik-api:chats:messages-create', args=[self.uri])


class ChatSessionMessage(TrackableDateModel):
    """Store messages for a session."""

    user = models.ForeignKey(User, on_delete=models.PROTECT)
    chat_session = models.ForeignKey(
        ChatSession, related_name='messages', on_delete=models.PROTECT
    )
    message = models.TextField(default='', max_length=2000)
    file_obj = models.ForeignKey(FileField, null=True, blank=True, on_delete=models.CASCADE, verbose_name=_('File'))

    def get_api_uri(self):
        return reverse('aparnik-api:chats:messages-details', args=[self.pk])


class ChatSessionMemberManager(models.Manager):
    def get_queryset(self):
        return super(ChatSessionMemberManager, self).get_queryset().select_related()


class ChatSessionMember(TrackableDateModel):
    """Store all users in a chat session."""

    chat_session = models.ForeignKey(
        ChatSession, related_name='members', on_delete=models.PROTECT
    )
    user = models.ForeignKey(User, on_delete=models.PROTECT)

    objects = ChatSessionMemberManager()

    def get_api_uri(self):
        return reverse('aparnik-api:chats:members-details', args=[self.pk])


def post_save_chat_session_message_receiver(sender, instance, created, *args, **kwargs):
    if created:
        from .tasks import send_chat_message
        send_chat_message.delay(instance.pk)


post_save.connect(post_save_chat_session_message_receiver, sender=ChatSessionMessage)


class ChatMessageNotificationManager(models.Manager):
    def get_queryset(self):
        return super(ChatMessageNotificationManager, self).get_queryset().select_related()

    def active(self):
        return self.get_queryset()

    def mark_as_read_chat_session(self, user, chat_session):
        count = self.unread_chat_session(user, chat_session).update(is_read=True)
        return count

    def unread_chat_session(self, user, chat_session):
        return self.active().filter(chat_message__chat_session=chat_session, is_read=False,
                                                        user=user)

    def unread_count_chat_session(self, user, chat_session):
        return self.unread_chat_session(user, chat_session).count()

    def unread(self, user):
        return self.active().filter(user=user, is_read=False)

    def unread_count(self, user):
        return self.active().filter(user=user, is_read=False).count()


class ChatMessageNotification(TrackableDateModel):
    """Store all users notification in a chat message."""

    chat_message = models.ForeignKey(
        ChatSessionMessage, related_name='message_notifications', on_delete=models.PROTECT
    )
    user = models.ForeignKey(User, on_delete=models.PROTECT)
    is_read = models.BooleanField(default=False, verbose_name=_('Is read'))

    objects = ChatMessageNotificationManager()




# Generated by Django 2.2 on 2020-04-05 16:27

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('aboutus', '0002_auto_20200107_1403'),
    ]

    operations = [
        migrations.AlterField(
            model_name='information',
            name='phone',
            field=models.CharField(blank=True, default='', max_length=15, verbose_name='Phone Number'),
        ),
    ]



# -*- coding: utf-8 -*-


from django.db import models
from django.db.models.signals import post_save
from django.core.validators import ValidationError
from django.contrib.auth import get_user_model
from django.utils.translation import ugettext_lazy as _
from django.utils.timezone import now

from dateutil.relativedelta import relativedelta
from decimal import Decimal

from aparnik.settings import aparnik_settings
from django.db.models import Sum
from aparnik.utils.utils import round
from aparnik.utils.formattings import formatprice
from aparnik.utils.fields import *
from aparnik.contrib.basemodels.models import BaseModel, BaseModelManager
from aparnik.contrib.bankaccounts.models import BankAccount

User = get_user_model()


# Create your models here.
class CoSaleManager(BaseModelManager):

    def get_queryset(self):
        return super(CoSaleManager, self).get_queryset()

    def active(self):
        return super(CoSaleManager, self).active().filter(is_active=True)

    def add_record(self, user_co_sale, user_bought, order):
        # TODO: how pay to user?
        # TODO: send notification
        cosale_obj = CoSale.objects.create(user_bought_obj=user_bought, user_co_sale_obj=user_co_sale, order_obj=order)
        return cosale_obj

    def change_order(self, order):
        try:
            co_sale = CoSale.objects.get(order_obj=order)
        except Exception as e:
            # نباید رکورد رو ادد کنیم بخاطر اینکه رکورد ها دارن از قسمت پیمنت های ثبت شده ادد میشن و
            # اگر اونجا ثبت نشدن یعنی پرداختی صورت نگرته !
            return CoSale.objects.none()
        # if not order.is_success and co_sale.status != CoSale.STATUS_ORDER_CANCEL:
        #     CoSaleHistory.objects.create(cosale_obj=co_sale, status=CoSale.STATUS_ORDER_CANCEL)
        # elif order.is_success and co_sale.status == CoSale.STATUS_ORDER_CANCEL:
        #     CoSaleHistory.objects.create(cosale_obj=co_sale, status=CoSale.STATUS_NOT_CLEARED)

    def summary(self, user):
        return CoSalePayment.objects.summary(user)

    def this_user(self, user, is_user_bought=False):
        if is_user_bought:
            return self.active().filter(user_bought_obj=user)
        return self.active().filter(user_co_sale_obj=user)

    def history(self, user, is_user_bought, start_date=None, end_date=None):
        from aparnik.packages.shops.orders.models import Order
        if not start_date:
            start_date = now().date()
        if not end_date:
            end_date = now()

        return self.this_user(user, is_user_bought).filter(Order.query_success('order_obj'),
                                                           Order.query_date('order_obj', start_date, end_date)).filter()

    def transaction_price(self, user, is_user_bought, start_date=None, end_date=None):
        if not start_date:
            start_date = now().date()
        if not end_date:
            end_date = now()
        return self.history(user, is_user_bought, start_date, end_date).aggregate(price=Sum('price'))['price'] or 0

    def transaction_price_string(self, user, is_user_bought, start_date=None, end_date=None):
        if not start_date:
            start_date = now().date()
            end_date = now()
        return '%s' % formatprice.format_price(
            self.history(user, is_user_bought, start_date, end_date).aggregate(price=Sum('price'))['price'] or 0)


class CoSale(BaseModel):
    STATUS_CLEARED = 'CL'
    STATUS_NOT_CLEARED = 'NC'
    STATUS_THE_REQUEST_FOR_SETTLEMENT_HAS_BEEN_RECEIVED = 'RSHBR'
    STATUS_ORDER_CANCEL = 'C'
    STATUS_TYPE = (
        # تسویه شده
        (STATUS_CLEARED, _('Cleared')),
        # تسویه نشده است
        (STATUS_NOT_CLEARED, _('Not Cleared')),
        # درخواست تسویه دریافت شده است
        (STATUS_THE_REQUEST_FOR_SETTLEMENT_HAS_BEEN_RECEIVED, _('The request for settlement has been received')),
        # سفارش کنسل شده است
        (STATUS_ORDER_CANCEL, _('Order Cancel')),
    )
    # کسی که خرید را انجام داده است.
    user_bought_obj = models.ForeignKey(User, related_name='cosale_user_bought', on_delete=models.CASCADE, verbose_name=_('User Bought'))
    # کسی که دعوت کرده است.
    user_co_sale_obj = models.ForeignKey(User, related_name='cosale_user_co_sale', on_delete=models.CASCADE, verbose_name=_('User Co Sale'))
    order_obj = models.ForeignKey('orders.Order', related_name='cosale_order', on_delete=models.CASCADE, verbose_name=_('Order'))
    is_active = models.BooleanField(default=True, verbose_name=_('Is Active'))
    price = PriceField(big=True, default=0, verbose_name=_('Price'))

    objects = CoSaleManager()

    def clean(self):
        pass

    def save(self, *args, **kwargs):
        self.full_clean()
        return super(CoSale, self).save(*args, **kwargs)

    def __str__(self):
        return str(self.order_obj)

    class Meta:
        verbose_name = _('Co Sale')
        verbose_name_plural = _('Co Sales')

    def calculate_share_price(self, user_co_sale):
        return (user_co_sale.co_sale_percentage / Decimal('100')) * self.order_obj.get_total_cost()

    @property
    def price_string(self):
        return '%s' % formatprice.format_price(self.price)

    # @staticmethod
    # def status_dictionary(status):
    #     for key, value in CoSale.STATUS_TYPE:
    #         if key == status:
    #             return {
    #                 'key': key,
    #                 'value': value
    #             }
    #     return None


def post_save_co_sale_receiver(sender, instance, created, *args, **kwargs):
    if created:
        instance.price = round(instance.calculate_share_price(instance.user_co_sale_obj))
        instance.save()


post_save.connect(post_save_co_sale_receiver, sender=CoSale)


class CoSalePaymentManager(BaseModelManager):

    def get_queryset(self):
        return super(CoSalePaymentManager, self).get_queryset()

    def active(self):
        return super(CoSalePaymentManager, self).active()

    def this_user(self, user):
        return self.active().filter(user_bank_account_obj__user_obj=user)

    def history(self, user, start_date=None, end_date=None):
        if not start_date:
            start_date = now().date()
        if not end_date:
            end_date = now()

        return self.this_user(user).filter()

    def transaction_price(self, user, start_date=None, end_date=None):
        if not start_date:
            start_date = now().date()
        if not end_date:
            end_date = now()
        return self.history(user, start_date, end_date).aggregate(price=Sum('price'))['price'] or 0

    def transaction_price_string(self, user, start_date=None, end_date=None):
        if not start_date:
            start_date = now().date()
            end_date = now()
        return '%s' % formatprice.format_price(
            self.history(user, start_date, end_date).aggregate(price=Sum('price'))['price'] or 0)

    # بستانکار
    def creditor_price(self, user):
        start_date = now() + relativedelta(years=-10)
        cosale_price = CoSale.objects.transaction_price(user, False, start_date)
        payment_price = CoSalePayment.objects.transaction_price(user, start_date)
        return cosale_price - payment_price

    def summary(self, user):
        dict = []

        price = CoSalePayment.objects.creditor_price(user)
        key = 'NC'

        dict.append({
            'price': price,
            'price_string': formatprice.format_price(price),
            'status':
                {
                    "value": "تسویه نشده",
                    "key": key
                },
        })

        for (key, value) in CoSalePayment.STATUS_TYPE:
            price = CoSalePayment.objects.active().filter(user_bank_account_obj__user_obj=user, status=key).aggregate(price=Sum('price'))[
                        'price'] or 0.0
            dict.append({
                'price': price,
                'price_string': formatprice.format_price(price),
                'status':
                    {
                        "value": value,
                        "key": key
                    },
            })

        return dict


class CoSalePayment(BaseModel):
    STATUS_CLEARED = 'CL'
    STATUS_THE_REQUEST_FOR_SETTLEMENT_HAS_BEEN_RECEIVED = 'RSHBR'
    STATUS_CANCEL = 'C'
    STATUS_TYPE = (
        # تسویه شده
        (STATUS_CLEARED, _('Cleared')),
        # درخواست تسویه دریافت شده است
        (STATUS_THE_REQUEST_FOR_SETTLEMENT_HAS_BEEN_RECEIVED, _('The request for settlement has been received')),
        # کنسل شده
        (STATUS_CANCEL, _('Cancel')),
    )

    price = PriceField(big=True, verbose_name=_('Price'))
    user_bank_account_obj = models.ForeignKey(BankAccount, on_delete=models.CASCADE, verbose_name=_('User Bank Account'))
    tracking_number = models.CharField(max_length=255, blank=True, null=True, verbose_name=_('Tracking Number'))
    status = models.CharField(max_length=10, choices=STATUS_TYPE,
                              default=STATUS_THE_REQUEST_FOR_SETTLEMENT_HAS_BEEN_RECEIVED,
                              verbose_name=_('Status'))

    objects = CoSalePaymentManager()

    def clean(self):
        if self.id:
            payment = CoSalePayment.objects.get(pk=self.id)
            if payment.price != self.price:
                raise ValidationError({'price': [_('You can\'t change price.')]})
        creditor = CoSalePayment.objects.creditor_price(self.user_bank_account_obj.user_obj)
        if self.price > creditor:
            raise ValidationError({'price': [_('Your price request is more than creditor.')]})

    def save(self, *args, **kwargs):
        self.full_clean()
        return super(CoSalePayment, self).save(*args, **kwargs)

    def __str__(self):
        return '%s' % self.id

    class Meta:
        verbose_name = _('Co Sale Payment')
        verbose_name_plural = _('Co Sale Payments')

    @property
    def price_string(self):
        return formatprice.format_price(self.price)



from django.conf.urls import url, include

from .views import SubscriptionListAPIView, SubscriptionOrderListAPIView, SubscriptionOrderActiveListAPIView

app_name = 'subscriptions'

urlpatterns = [
    url(r'^$', SubscriptionListAPIView.as_view(), name='list'),
    url(r'^mine-history/$', SubscriptionOrderListAPIView.as_view(), name='list-mine'),
    url(r'^mine-active/$', SubscriptionOrderActiveListAPIView.as_view(), name='list-active'),
]



# -*- coding: utf-8 -*-


from django.contrib.auth import get_user_model
from django.db import models
from django.utils.timezone import now
from django.utils.translation import ugettext_lazy as _

from aparnik.packages.shops.orders.models import Order
from aparnik.packages.shops.products.models import Product, ProductManager

User = get_user_model()


# Create your models here.
class SubscriptionManager(ProductManager):

    def get_queryset(self):
        return super(SubscriptionManager, self).get_queryset()

    def active(self, user=None):
        return super(SubscriptionManager, self).active(user=user)


class Subscription(Product):
    TYPE_ALL = 'a'
    TYPE_CUSTOM = 'c'
    TYPE_CHOICES = (
        (TYPE_ALL, _('All')),
        (TYPE_CUSTOM, _('Custom')),
    )

    type = models.CharField(max_length=5, default=TYPE_ALL, choices=TYPE_CHOICES, verbose_name=_('Type'))
    duration = models.DurationField(default='30 00:00:00', verbose_name=_('Duration'))
    description = models.TextField(blank=True, null=True, verbose_name=_('Description'))
    orders = models.ManyToManyField('orders.Order', through='SubscriptionOrder', verbose_name=_('Orders'))
    products = models.ManyToManyField('products.Product', blank=True, related_name='subscriptions',
                                      verbose_name=_('Products'))

    objects = SubscriptionManager()

    def buy_success(self, order):
        expire_at = now() + self.duration
        try:
            subscription_order = SubscriptionOrder.objects.get(subscription_obj=self, order_obj=order)
            subscription_order.expire_at = expire_at
            subscription_order.save()
        except:
            SubscriptionOrder.objects.create(
                subscription_obj=self,
                order_obj=order,
                expire_at=expire_at,
            )
        return super(Subscription, self).buy_success(order)

    def is_buy(self, user):

        is_buy = super(Subscription, self).is_buy(user=user)

        if is_buy:
            return self.subscriptionorder_set.active().exists()

        return False

    def price_string_description(self, user):

        if self.is_buy(user):
            days_remain = abs((now() - self.subscriptionorder_set.active().first().expire_at).days)
            return '%s روز دیگر باقی مانده' % days_remain
        return super(Subscription, self).price_string_description(user=user)

    class Meta:
        verbose_name = _('Subscription')
        verbose_name_plural = _('Subscriptions')

    def clean(self):
        # if self.type == self.TYPE_ALL and self.products.count() > 0:
        #     raise ValidationError({
        #         'type': [_(
        #             'You can not choice product when select the subscription for all products.'
        #         )]})
        pass

    def save(self, *args, **kwargs):
        self.full_clean()
        return super(Subscription, self).save(*args, **kwargs)


class SubscriptionOrderManager(models.Manager):

    def get_queryset(self):
        return super(SubscriptionOrderManager, self).get_queryset()

    def active(self):
        return self.get_queryset().filter(
            expire_at__gte=now()
        )

    def mine(self, user):
        return self.active().filter(
            Order.query_success('order_obj', user=user),
        )

    def mine_history(self, user):
        return self.get_queryset().filter(
            Order.query_success(prefix='order_obj', user=user),
        )


class SubscriptionOrder(models.Model):
    subscription_obj = models.ForeignKey(Subscription, on_delete=models.CASCADE, verbose_name=_('Subscription'))
    order_obj = models.ForeignKey('orders.Order', on_delete=models.CASCADE, verbose_name=_('Order'))
    expire_at = models.DateTimeField(auto_now_add=False, auto_now=False, verbose_name=_('Expire at'))

    created_at = models.DateTimeField(auto_now_add=True, auto_now=False, verbose_name=_('Created at'))
    update_at = models.DateTimeField(auto_now_add=False, auto_now=True, verbose_name=_('Update at'))

    objects = SubscriptionOrderManager()

    def __str__(self):
        return "%s" % self.id

    class Meta:
        verbose_name = _('Subscription Order')
        verbose_name_plural = _('Subscriptions Orders')



from django.conf.urls import url, include

from .views import NotificationListAPIView, NotificationReadAPIView, NotificationDetailAPIView

app_name = 'notifications'

urlpatterns = [
    url(r'^$', NotificationListAPIView.as_view(), name='list'),
    url(r'^(?P<id>\d+)/read/$', NotificationReadAPIView.as_view(), name='read'),
    url(r'^(?P<id>\d+)/$', NotificationDetailAPIView.as_view(), name='detail'),
    url(r'^read/$', NotificationReadAPIView.as_view(), name='reads-all'),
]


from django.urls import reverse
from rest_framework import serializers

from aparnik.packages.shops.products.api.serializers import ProductListSerializer, ProductDetailSerializer
from ..models import Subscription, SubscriptionOrder


class SubscriptionListSerializer(ProductListSerializer):

    class Meta:
        model = Subscription
        fields = ProductListSerializer.Meta.fields + [
            'type',
            'duration',
            'description',
        ]


class SubscriptionDetailSerializer(SubscriptionListSerializer, ProductDetailSerializer):

    class Meta:
        model = Subscription
        fields = SubscriptionListSerializer.Meta.fields + ProductDetailSerializer.Meta.fields + [

        ]


# Subscription order
class SubscriptionOrderListSerializer(serializers.ModelSerializer):
    subscription = serializers.SerializerMethodField()

    class Meta:
        model = SubscriptionOrder
        fields = [
            'pk',
            'subscription',
            'expire_at',
            'created_at',
            'update_at',
        ]

    def get_subscription(self, obj):
        return SubscriptionListSerializer(
            obj.subscription_obj,
            many=False,
            read_only=True,
            context=self.context
        ).data
